
library (gsl)
library (akima)

if (1|| !exists ("qrngMat")) {
  q <- qrng_alloc (type = c("sobol"), dim=4)
  qrngSize <- 1024
  qrngMat <- qrng_get (q, qrngSize)
  qrngMatTrans <- t(qrngMat)
}

if (!exists("DoNotLoadDLL") && !is.loaded("calcExpPay")) {
  dllFileName <- "../calc-strat.so"
#  dllFileName <- "calc/calc-strat.so"
  dyn.load (dllFileName)
}


lastIndivLL <- rep (0, AnzSubj)
lastCLC <- rep (0, AnzSubj)
lastPostClass <- array (0, c(AnzSubj,1))
savedCLC <- rep (0, 10)

oldVal <- -100
GetLLCt <- 0

FixedParsFL <- c()
FixedParsVals <- c()

if (!exists ("RelAnzTypes")) RelAnzTypes <- 1:1

#############################################################################################
#############################################################################################

GetProminenceLevs <- function (treat, FixPromZero=-10) {

    # copy this to the highest-level input scripts
    # check that it works correctly
    # create also homogeneous het-logit
    # check base-names everywhere and optim-scripts that start from these base-estimates

    

    irName <- paste("prLevs", treat, sep="")
    if ( eval(parse(text = paste("exists(\"", irName, "\")", sep="")))) {
        eval(parse(text = paste(" prLevs <- ", irName)))
        return (prLevs)
    }

#    cat ("Var ", irName, " does not exist\n")

    AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
    transV <- (1:AnzStrats-1)*Step[treat] + Low[treat]

    prFak <- c(.1, .5, 1, 5, 10, 50, 100, 500, 1000)

    prLevs <- rep (-2, AnzStrats)
    stratsLeft <- 1:AnzStrats

    ct <- 6
    first <- TRUE
    indZero <- (transV> -0.01) & (transV< 0.01)

    if (FixPromZero > -10 && sum(indZero)>0) {
        prLevs[indZero] <- FixPromZero
        zeroStrats <- stratsLeft[indZero]
        stratsLeft <- setdiff (stratsLeft, zeroStrats)
        first <- FALSE
    }
    
    for (f in rev(prFak)) {

        s <- stratsLeft[transV[stratsLeft] %% f == 0]

#        cat ("Faktor", f, ":", transV[s], "\n")

        if (length(s) > 1*first) {
            prLevs[s] <- ct
            stratsLeft <- setdiff (stratsLeft, s)
            first <- FALSE
            if (length(stratsLeft)==0) break
        }

#        cat ("prLevs:", prLevs, "\n\n")

        ct <- ct-1
    }

    eval(parse(text = paste(irName, " <<-  prLevs")))
    
#    cat ("transV:", transV, "\n")
#    cat ("prLevs:", prLevs, "\n\n")

    return (prLevs)
}


PayoffProf <- function (strat, subj, treat) {
       # Careful! everywhere else "subj" is the lineNr, thus this code is broken
       # Does not really matter, as evaluation is in cuda
    
    1 + c( RefData$Up[(subj-1)*AnzTreat+treat]-strat+1, (strat-1) * RefData$Rate[(subj-1)*AnzTreat+treat] )
    if (Moderate==2) pay <- pay/30      # Conversion to Dollar

    return (pay)
}


ULinAltr <- function (strat, alpha, subj, treat) {
  pay <- PayoffProf(strat, subj, treat)
  util <- pay[1] + alpha * pay[2]
  return (util)
}

UCDAltr <- function (strat, alpha, subj, treat) {
  pay <- pmax(0.001, PayoffProf(strat, subj, treat))
  util <- pay[1]^(1-alpha) * pay[2]^alpha
  return (util)
}

UCESAltr <- function (strat, alpha, subj, treat) {
  if (abs(alpha[2])<1e-10) return (UCDAltr (strat, alpha[1], subj, treat))

  pay <- pmax(0.001, PayoffProf(strat, subj, treat))
  s <- (1-alpha[1]) * pay[1]^alpha[2] + alpha[1] * pay[2]^alpha[2]
#  cat (pay, " => ", (1-alpha[1]) * pay[1]^alpha[2], "+", alpha[1] * pay[2]^alpha[2], "=", s, "\n")
  if (s<=0) return (-pmax(0.001,abs(s))^(1/alpha[2])) else return (s^(1/alpha[2]))
}

if (exists("UCESAltrV")) rm (UCESAltrV)

ConvertToHighPrec <- function (x) {  #Input: log-scale
  y <- rep (0, 2)

  y[2] <- round ( x / log(10) )
  y[1] <- exp( x - y[2]*log(10))

#  if (x>1.0e16) return (c(1,x / log(10)))
  return (y)
}

ConvertToLogScale <- function (y) { #Input: High precision

  x <- y[2] * log(10) + log(y[1])
  
  return (x)
}

ConvertToRealScale <- function (y) { #Input: High precision

  x <- y[1] * 10^y[2]
  
  return (x)
}

AddHighPrec <- function (x1, x2) {
  y <- rep (0, 2)

  if (x1[1]==0) return (x2)
  if (x2[1]==0) return (x1)

  if (x1[2]>=x2[2]) {
    y[1] <- x1[1] + x2[1] * 10^(x2[2]-x1[2])
    y[2] <- x1[2]
  } else {
    y[1] <- x2[1] + x1[1] * 10^(x1[2]-x2[2])
    y[2] <- x2[2]
  }
  
  return (y)
}

DivideHighPrec <- function (x1, x2) {
  y <- rep (0, 2)

  y[2] <- x1[2] - x2[2]
  y[1] <- x1[1] / x2[1]
  
  return (y)
}

MultiplyHighPrec <- function (x1, x2) {
  y <- rep (0, 2)

  y[2] <- x1[2] + x2[2]
  y[1] <- x1[1] * x2[1]
  
  return (y)
}


#######################################################################################
#######################################################################################
#######################################################################################


NormalizeTypeShares <- function (pars) {
  shares <- pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]

  if (AnzTypes > 1) {
    if (sum(shares[2:length(shares)]) > 1) {
      shares[1] <- 0
      shares <- shares/sum(shares)
    } else {
      shares[1] <- 1 - sum(shares[2:length(shares)])
    }
  } else {
    shares <- 1
  }
  
  pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)] <- shares 

  return (pars)
}

GetIndivTypeStrat <- function (pars, t, subj, treat) {
    
#  share <- pars[(t-1)*(AnzTypePars+1) + 1]
  TypePars <- pars[(t-1)*(AnzTypePars+1) + 1 + 1:AnzTypePars]
  strat <- GetTypeStrat (TypePars, subj, treat)

  if (sum(strat)>1.01) cat ("\n\nXXX Pars:", pars, "\n  =>  Sum =", sum(strat), "  (", strat, ")\n\n")

  return (strat)
}

if (!exists("MixMode")) MixMode <- 0
lastStratMoms <- array (0, c(50, 10))
lastStratLength <- rep (0, 50)
lastStratM <- 0

truncQnorm <- function (q, m, sd) {
    if (pnorm (-0.5, m=m, sd=sd)>1-1e-2) m <- -0.5
    if (pnorm (0.5, m=m, sd=sd)<1e-2) m <- 0.5
    
    if (sd>1e-3) q2 <- pnorm (-0.5, m=m, sd=sd) + q * (pnorm (0.5, m=m, sd=sd) - pnorm (-0.5, m=m, sd=sd))
    else { q2 <- q*0+0.5 ; sd <- 1e-2 }

#    for (i in 1:length(q)) cat ("q", q[i], "-> q2", q2[i], "\t m =", m, "\tsd =", sd, "\t=>", qnorm (q2[i], m=m, sd=sd), "\n")
    
    return (qnorm (q2, m=m, sd=sd))
}

GetLL <- function (pars, RelTr=RelTreats) {
  pars <- pars * parFac
  if (length(pars)==length(FixedParsFL)) pars[FixedParsFL] <- FixedParsVals[FixedParsFL]
  
  ParMin <- rep(baseParMin, AnzTypes)
  ParMax <- rep(baseParMax, AnzTypes)

  penFac <- 1 + (sum(pmax (0, pars-ParMax)) - sum(pmin (0, pars-ParMin)))^2
  if (GetLLLog2)
    cat ("GetLL: ", " (", round(pars, digits=5), ")  =>  penFac =", penFac, "\n")

  pars <- pmin (ParMax, pmax(ParMin, pars))
  pars <- NormalizeTypeShares (pars)

  CallArgs <- paste(c(RelTr, AnzTypes, pars), collapse=" ")
  pos <- match (CallArgs, GetLLCallArgs)
  if (!is.na(pos)) {
    ret <- GetLLCallVals[pos]
    LL <- -ret/penFac
    if (GetLLLog2) cat (c("GetLL: ", " (", round(pars, digits=5), ")"))
    if (GetLLLog2) cat (c("  \t=>  LL =", round(LL, digits=5), "  ( ret =", round(ret, digits=5), ")\n"))

    return (ret)
  }
    
  if (GetLLLog || GetLLCt %% 5000==0) cat (c("GetLL (", GetLLCt, "): ", " (", paste (round(pars, digits=5), collapse=","), ")"))

  AnzSimul <- 1024
  if (!exists ("extCallScheme")) extCallScheme <- 1
  IndivLL <- array (1, c(AnzSubj, AnzTypes, AnzSimul))
  for (type in 1:AnzTypes) {
      if (exists("MixTypesV") && (type %in% MixTypesV || type == AnzTypes)) {
          if (MixMode %in% 2:3) {
              subsPars <- rep(0, 3*AnzSimul)
              d <- pars[c(2,5:9) + (type-1) * (AnzTypePars+1)]
              if (MixMode==2) subsPars[3*1:AnzSimul-2] <- pmax(0, qnorm (qrngMat[1:AnzSimul,1], m=d[1], sd=abs(d[6])))
              else if (MixMode==3) subsPars[3*1:AnzSimul-2] <- pmax(0, exp(qnorm (qrngMat[1:AnzSimul,1], m=d[1], sd=abs(d[6]))))
              subsPars[3*1:AnzSimul-1] <- truncQnorm (qrngMat[1:AnzSimul,2], m=d[2], sd=abs(d[4]))
              subsPars[3*1:AnzSimul] <- pmax (-10, qnorm (qrngMat[1:AnzSimul,3], m=d[3], sd=abs(d[5])))
          } else if (MixMode %in% 4:5) {
              subsPars <- rep(0, 4*AnzSimul)
              d <- pars[c(2,5:9) + (type-1) * (AnzTypePars+1)]
              d2 <- pars[c(3:4) + (type-1) * (AnzTypePars+1)]
              
              if (MixMode==4) {
                  subsPars[4*1:AnzSimul-3] <- pmax(0, exp(qnorm (qrngMat[1:AnzSimul,1], m=d[1], sd=abs(d[6]))))
                  subsPars[4*1:AnzSimul-2] <- pmax(0, exp(qnorm (qrngMat[1:AnzSimul,4], m=d2[1], sd=abs(d2[2]))))
                  subsPars[4*1:AnzSimul-1] <- truncQnorm (qrngMat[1:AnzSimul,2], m=d[2], sd=abs(d[4]))
                  subsPars[4*1:AnzSimul] <- pmax (-10, qnorm (qrngMat[1:AnzSimul,3], m=d[3], sd=abs(d[5])))
              } else if (MixMode==5) {
                  subsPars[4*1:AnzSimul-3] <- pmax(0, d[1])
                  subsPars[4*1:AnzSimul-2] <- pmax(0, exp(qnorm (qrngMat[1:AnzSimul,1], m=d2[1], sd=abs(d2[2]))))
                  subsPars[4*1:AnzSimul-1] <- truncQnorm (qrngMat[1:AnzSimul,2], m=d[2], sd=abs(d[4]))
                  subsPars[4*1:AnzSimul] <- pmax (-10, qnorm (qrngMat[1:AnzSimul,3], m=d[3], sd=abs(d[5])))

                  lastAlpQ <<- qrngMat[1:AnzSimul,2]
              }
          } else {
              subsPars <- rep(0, 2*AnzSimul)
              d <- pars[5:8 + (type-1) * (AnzTypePars+1)]
              subsPars[2*1:AnzSimul-1] <- truncQnorm (qrngMat[1:AnzSimul,1], m=d[1], sd=abs(d[3]))
              subsPars[2*1:AnzSimul] <- pmax (-10, qnorm (qrngMat[1:AnzSimul,2], m=d[2], sd=abs(d[4])))
              
              lastAlpQ <<- qrngMat[1:AnzSimul,1]
          }
          
          if (1) {
              tPars <- c(pars[2:6 + (type-1) * (AnzTypePars+1)], dim(RefData)[1], AnzSimul, round(length(subsPars)/AnzSimul), extCallScheme, Moderate)
              choiceProbs <- rep (0, dim(RefData)[1]*AnzSimul)
              lastSubsPars <<- subsPars
              out <<- .C("calcExpPay",
                         data=as.double(TransData),
                         pars=as.double(tPars),
                         subsPars=as.double(subsPars),
                         choiceProbs=as.double(choiceProbs))
              pr <- array (out$choiceProbs, c(dim(RefData)[1], AnzSimul))
              
#              for (s in 1:AnzSimul) 
              for (treat in RelTr)
                  for (subj in 1:AnzSubj) {
                      v <- pr[(subj-1)*AnzTreat+treat,]
                      v[is.nan(v)] <- 1e-38
                      IndivLL[subj,type,] <- IndivLL[subj,type,] * v
                  }
          } else {
              pars2 <- pars
              for (s in 1:AnzSimul) {
                  pars2[5 + (type-1)*(AnzTypePars+1)] <- subsPars[2*s-1]
                  pars2[6 + (type-1)*(AnzTypePars+1)] <- subsPars[2*s]
              
                  for (treat in RelTr)
                      for (subj in 1:AnzSubj) {
                          StratProbs <- GetIndivTypeStrat (pars2, type, subj, treat)
                          NormStrat <- RefData$NormPass[(subj-1)*AnzTreat+treat] + 1
                          IndivLL[subj,type,s] <- IndivLL[subj,type,s] * StratProbs[NormStrat]
                      }
              }
          }
      } else {
          if (1) {
              tPars <- pars[2:6 + (type-1) * (AnzTypePars+1)]
              if (MixMode %in% 2:5) subsPars <- tPars[c(1,4:5)] else subsPars <- tPars[4:5]
              cPars <- c(tPars, dim(RefData)[1], 1, length(subsPars), extCallScheme, Moderate)
              choiceProbs <- rep (0, dim(RefData)[1])
              out <<- .C("calcExpPay",
                         data=as.double(TransData),
                         pars=as.double(cPars),
                         subsPars=as.double(subsPars),
                         choiceProbs=as.double(choiceProbs))
              pr <- out$choiceProbs
              
#              print (RelTr)
              for (treat in RelTr)
                  for (subj in 1:AnzSubj)
                      IndivLL[subj,type,] <- IndivLL[subj,type,] * pr[(subj-1)*AnzTreat+treat]
          } else {
            for (treat in RelTr)
              for (subj in 1:AnzSubj) {
                StratProbs <- GetIndivTypeStrat (pars, type, subj, treat)
                NormStrat <- RefData$NormPass[(subj-1)*AnzTreat+treat] + 1
                IndivLL[subj,type,] <- IndivLL[subj,type,] * StratProbs[NormStrat]
              }
          }
      }
  }

  IndivLL2 <- array (1, c(AnzSubj, AnzTypes))
  lastIndivLLFull <<- IndivLL
  for (type in 1:AnzTypes)
      for (subj in 1:AnzSubj) {
          v <- IndivLL[subj,type,]
          v[is.nan(v)] <- 1e-308
          IndivLL2[subj,type] <- mean (v)
      }
  
  LL <- 0
  lastIndivLL2 <<- IndivLL2
  lastIndivLL <<- rep (0, AnzSubj)
  shares <- pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]
  lastPostClass <<- array (0, c(AnzSubj,AnzTypes))
  for (subj in 1:AnzSubj) {
    LL <- LL + max (-500, log (sum (shares * IndivLL2[subj,])))
    lastIndivLL[subj] <<- log (sum (shares * IndivLL2[subj,]))
    lastPostClass[subj,] <<- shares * IndivLL2[subj,] / max (1e-300, sum (shares * IndivLL2[subj,]))
  }
  
  ret <- -LL*penFac
  if (GetLLLog || GetLLLog2 || GetLLCt %% 5000==0)
    cat (c("  \t=>  LL =", round(LL, digits=5), "  ( ret =", round(ret, digits=5), ")\n"))
  else if (0 && GetLLLog) cat (round(LL, digits=3), "\n")

  GetLLCt <<- GetLLCt + 1
  
  if (FLSaveRes) {
    GetLLCallArgs <<- c(GetLLCallArgs, CallArgs)                  
    GetLLCallVals <<- c(GetLLCallVals, ret)
    save (GetLLCallArgs, GetLLCallVals, file = DataFile)
  }

  return (ret)
}

GetLL2 <- function (pars, GetUtil=UAltr, RelTr=RelTreats) { # nutzt lastPostClass[subj,] als weights
  pars <- pars * parFac
  if (length(pars)==length(FixedParsFL)) pars[FixedParsFL] <- FixedParsVals[FixedParsFL]
  
  ParMin <- rep(baseParMin, AnzTypes)
  ParMax <- rep(baseParMax, AnzTypes)

  pars <- pmin (ParMax, pmax(ParMin, pars))
  pars <- NormalizeTypeShares (pars)

  AnzSimul <- 1024
  if (!exists ("extCallScheme")) extCallScheme <- 1
  IndivLL <- array (1, c(AnzSubj, AnzTypes, AnzSimul))
  for (type in 1:AnzTypes) {
      if (exists("MixTypesV") && (type %in% MixTypesV || type == AnzTypes)) {
          if (MixMode %in% 2:3) {
              subsPars <- rep(0, 3*AnzSimul)
              d <- pars[c(2,5:9) + (type-1) * (AnzTypePars+1)]
              if (MixMode==2) subsPars[3*1:AnzSimul-2] <- pmax(0, qnorm (qrngMat[1:AnzSimul,1], m=d[1], sd=abs(d[6])))
              else if (MixMode==3) subsPars[3*1:AnzSimul-2] <- pmax(0, exp(qnorm (qrngMat[1:AnzSimul,1], m=d[1], sd=abs(d[6]))))
              subsPars[3*1:AnzSimul-1] <- truncQnorm (qrngMat[1:AnzSimul,2], m=d[2], sd=abs(d[4]))
              subsPars[3*1:AnzSimul] <- pmax (-10, qnorm (qrngMat[1:AnzSimul,3], m=d[3], sd=abs(d[5])))
          } else if (MixMode %in% 4:5) {
              subsPars <- rep(0, 4*AnzSimul)
              d <- pars[c(2,5:9) + (type-1) * (AnzTypePars+1)]
              d2 <- pars[c(3:4) + (type-1) * (AnzTypePars+1)]
              
              if (MixMode==4) {
                  subsPars[4*1:AnzSimul-3] <- pmax(0, exp(qnorm (qrngMat[1:AnzSimul,1], m=d[1], sd=abs(d[6]))))
                  subsPars[4*1:AnzSimul-2] <- pmax(0, exp(qnorm (qrngMat[1:AnzSimul,4], m=d2[1], sd=abs(d2[2]))))
                  subsPars[4*1:AnzSimul-1] <- truncQnorm (qrngMat[1:AnzSimul,2], m=d[2], sd=abs(d[4]))
                  subsPars[4*1:AnzSimul] <- pmax (-10, qnorm (qrngMat[1:AnzSimul,3], m=d[3], sd=abs(d[5])))
              } else if (MixMode==5) {
                  subsPars[4*1:AnzSimul-3] <- pmax(0, d[1])
                  subsPars[4*1:AnzSimul-2] <- pmax(0, exp(qnorm (qrngMat[1:AnzSimul,1], m=d2[1], sd=abs(d2[2]))))
                  subsPars[4*1:AnzSimul-1] <- truncQnorm (qrngMat[1:AnzSimul,2], m=d[2], sd=abs(d[4]))
                  subsPars[4*1:AnzSimul] <- pmax (-10, qnorm (qrngMat[1:AnzSimul,3], m=d[3], sd=abs(d[5])))
              }
          } else {
              subsPars <- rep(0, 2*AnzSimul)
              d <- pars[5:8 + (type-1) * (AnzTypePars+1)]
              subsPars[2*1:AnzSimul-1] <- truncQnorm (qrngMat[1:AnzSimul,1], m=d[1], sd=abs(d[3]))
              subsPars[2*1:AnzSimul] <- pmax (-10, qnorm (qrngMat[1:AnzSimul,2], m=d[2], sd=abs(d[4])))
          }
          
          if (1) {
              tPars <- pars[2:6 + (type-1) * (AnzTypePars+1)]
              cPars <- c(tPars, dim(RefData)[1], AnzSimul, round(length(subsPars)/AnzSimul), extCallScheme, Moderate)
              choiceProbs <- rep (0, dim(RefData)[1]*AnzSimul)
              out <<- .C("calcExpPay",
                         data=as.double(TransData),
                         pars=as.double(cPars),
                         subsPars=as.double(subsPars),
                         choiceProbs=as.double(choiceProbs))
              pr <- array (out$choiceProbs, c(dim(RefData)[1], AnzSimul))
              
#              for (s in 1:AnzSimul) 
              for (treat in RelTr)
                  for (subj in 1:AnzSubj) {
                      v <- pr[(subj-1)*AnzTreat+treat,]
                      v[is.nan(v)] <- 1e-38
                      IndivLL[subj,type,] <- IndivLL[subj,type,] * v
                  }
          } else {
              pars2 <- pars
              for (s in 1:AnzSimul) {
                  pars2[5 + (type-1)*(AnzTypePars+1)] <- subsPars[2*s-1]
                  pars2[6 + (type-1)*(AnzTypePars+1)] <- subsPars[2*s]
              
                  for (treat in RelTr)
                      for (subj in 1:AnzSubj) {
                          StratProbs <- GetIndivTypeStrat (pars2, type, subj, treat)
                          NormStrat <- RefData$NormPass[(subj-1)*AnzTreat+treat] + 1
                          IndivLL[subj,type,s] <- IndivLL[subj,type,s] * StratProbs[NormStrat]
                      }
              }
          }
      } else {
          if (1) {
              tPars <- pars[2:6 + (type-1) * (AnzTypePars+1)]
              if (MixMode %in% 2:5) subsPars <- tPars[c(1,4:5)] else subsPars <- tPars[4:5]
              cPars <- c(tPars, dim(RefData)[1], 1, length(subsPars), extCallScheme, Moderate)
              choiceProbs <- rep (0, dim(RefData)[1])
              out <<- .C("calcExpPay",
                         data=as.double(TransData),
                         pars=as.double(cPars),
                         subsPars=as.double(subsPars),
                         choiceProbs=as.double(choiceProbs))
              pr <- out$choiceProbs
              
              for (treat in RelTr)
                  for (subj in 1:AnzSubj)
                      IndivLL[subj,type,] <- IndivLL[subj,type,] * pr[(subj-1)*AnzTreat+treat]
          } else 
              for (treat in RelTr)
                  for (subj in 1:AnzSubj) {
                      StratProbs <- GetIndivTypeStrat (pars, type, subj, treat)
                      NormStrat <- RefData$NormPass[(subj-1)*AnzTreat+treat] + 1
                      IndivLL[subj,type,] <- IndivLL[subj,type,] * StratProbs[NormStrat]
                  }
      }
  }

  IndivLL2 <- array (1, c(AnzSubj, AnzTypes))
  for (type in 1:AnzTypes)
      for (subj in 1:AnzSubj) {
          v <- IndivLL[subj,type,]
          v[is.nan(v)] <- 1e-308
          IndivLL2[subj,type] <- mean (v)
      }
  
  LL <- 0
  shares <- pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]
  for (subj in 1:AnzSubj) {
      iLL <- lastPostClass[subj,] * IndivLL2[subj,]
      LL <- LL + max (-500, log (sum (iLL)))
      lastIndivLL[subj] <<- log (sum (iLL))
      lastPostClass[subj,] <<- iLL / sum (iLL)
  }
  
  ret <- -LL

  return (ret)
}

calcLLmax <- function (RelTr) {
    
  AnzClasses <- 4

  indivZeroes <- array (0, c(AnzSubj))
  for (subj in 1:AnzSubj) indivZeroes[subj] <- sum(RefData$NormPass[(subj-1)*AnzTreat+1:AnzTreat]==0)
  indivClass <- 1 + pmin (indivZeroes, AnzClasses-1)
  shares <- rep (0, AnzClasses)
  for (cla in 1:AnzClasses) shares[cla] <- sum(indivClass==cla)/length(indivClass)

  lastIndivZeroes <<- indivZeroes
  lastIndivClass <<- indivClass
  lastShares <<- shares
      
  IndivLL <- array (0, c(AnzSubj))
  for (treat in RelTr) {
      AllStrat <- RefData$NormPass[(1:AnzSubj)*AnzTreat+treat] + 1
      
      for (cla in 1:AnzClasses) {
          subjSet <- (1:AnzSubj)[indivClass==cla]
          AllStratC <- RefData$NormPass[(subjSet-1)*AnzTreat+treat] + 1

          for (subj in subjSet) {
              if (IndivLL[subj]==0) IndivLL[subj] <- log(shares[cla])
          
              NormStrat <- RefData$NormPass[(subj-1)*AnzTreat+treat] + 1
              stratPr <- sum(NormStrat == AllStratC) / length(AllStratC)
              IndivLL[subj] <- IndivLL[subj] + log (stratPr)
              
#              cat ("Subj", subj, "\tcla", cla, ":", NormStrat, "=>", stratPr, IndivLL[subj], "\n")
          }
      }

  }

  if (0 && numMax0>0)  {
      IndivLL[indivMax==0] <- log (numMax0)
      IndivLL[indivMax!=0] <- log (1-numMax0) + IndivLL[indivMax!=0]
  }

  lastIndivLL <<- IndivLL
  return (sum(IndivLL))
}

calcLLmin <- function (RelTr) {

  IndivLL <- array (0, c(AnzSubj))
  for (treat in RelTr)
      for (subj in 1:AnzSubj) {
          stratPr <- 1 / (RefData$Up[(subj-1)*AnzTreat+treat])  # AnzStrats
          IndivLL[subj] <- IndivLL[subj] + log (stratPr)
      }

  lastIndivLL <<- IndivLL
  return (sum(IndivLL))
}

source ("../print-input.R")
