
library (gsl)
library (akima)

if (1|| !exists ("qrngMat")) {
  q <- qrng_alloc (type = c("sobol"), dim=4)
  qrngSize <- 1024
  qrngMat <- qrng_get (q, qrngSize)
  qrngMatTrans <- t(qrngMat)
}


lastIndivLL <- rep (0, AnzSubj)
lastCLC <- rep (0, AnzSubj)
lastPostClass <- array (0, c(AnzSubj,1))
savedCLC <- rep (0, 10)

oldVal <- -100
GetLLCt <- 0

FixedParsFL <- c()
FixedParsVals <- c()

if (!exists ("RelAnzTypes")) RelAnzTypes <- 1:1

#############################################################################################
#############################################################################################

GetProminenceLevs <- function (treat, subj=1, FixPromZero=-10) {

    # copy this to the highest-level input scripts
    # check that it works correctly
    # create also homogeneous het-logit
    # check base-names everywhere and optim-scripts that start from these base-estimates

    

    irName <- paste("prLevs", treat, sep="")
    if ( eval(parse(text = paste("exists(\"", irName, "\")", sep="")))) {
        eval(parse(text = paste(" prLevs <- ", irName)))
        return (prLevs)
    }

#    cat ("Var ", irName, " does not exist\n")

    AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
    
#    transV <- (1:AnzStrats-1)*Step[treat] + Low[treat]     # number of ECU deducted from own account
    transV <- 1:AnzStrats-1                                 # number of tokens entered to be transferred in experiment

    prFak <- c(.1, .5, 1, 5, 10, 50, 100, 500, 1000)

    prLevs <- rep (-2, AnzStrats)
    stratsLeft <- 1:AnzStrats

    ct <- 6
    first <- TRUE
    indZero <- (transV> -0.01) & (transV< 0.01)

    if (FixPromZero > -10 && sum(indZero)>0) {
        prLevs[indZero] <- FixPromZero
        zeroStrats <- stratsLeft[indZero]
        stratsLeft <- setdiff (stratsLeft, zeroStrats)
        first <- FALSE
    }
    
    for (f in rev(prFak)) {

        s <- stratsLeft[transV[stratsLeft] %% f == 0]

#        cat ("Faktor", f, ":", transV[s], "\n")

        if (length(s) > 1*first) {
            prLevs[s] <- ct
            stratsLeft <- setdiff (stratsLeft, s)
            first <- FALSE
            if (length(stratsLeft)==0) break
        }

#        cat ("prLevs:", prLevs, "\n\n")

        ct <- ct-1
    }

    eval(parse(text = paste(irName, " <<-  prLevs")))
    
#    cat ("transV:", transV, "\n")
#    cat ("prLevs:", prLevs, "\n\n")

    return (prLevs)
}

PayoffProf <- function (strat, treat) {
  trans <- (strat-1)*Step[treat] + Low[treat]
  receiv <- trans * Rate[treat]
  pay <- 1 + c(DictEndow[treat]-trans, RecEndow[treat]+receiv) / ifelse (Moderate==2, 10, 1)

  return (pay)
}

ULinAltr <- function (strat, alpha, subj=1, treat) {
  pay <- PayoffProf(strat, treat)
  util <- (1-alpha[1]) * pay[1] + alpha[1] * pay[2]
  return (util)
}

UCDAltr <- function (strat, alpha, subj=1, treat) {
  pay <- pmax(0.001, PayoffProf(strat, treat))
  util <- pay[1]^(1-alpha[1]) * pay[2]^alpha[1]
  return (util)
}

UCESAltr <- function (strat, alpha, subj=1, treat) {
  if (abs(alpha[2])<1e-7) return (UCDAltr (strat, alpha[1], treat))

  pay <- pmax(0.001, PayoffProf(strat, treat))
  s <- (1-alpha[1]) * pay[1]^alpha[2] + alpha[1] * pay[2]^alpha[2]
  if (is.na(s)) {
#    cat ("pay =", pay, "\talpha =", alpha)
#    cat (" => ", (1-alpha[1]) * pay[1]^alpha[2], "+", alpha[1] * pay[2]^alpha[2], "=", s, "\n")

    s <- pmin (1e6, (1-alpha[1]) * pay[1]^alpha[2]) + pmin(1e6, alpha[1] * pay[2]^alpha[2])
  }
  if (s<=0) return (-pmax(0.001,abs(s))^(1/alpha[2])) else return (s^(1/alpha[2]))
}

UCESAltrV <- function (stratV, alpha, subj, treat) {

    trans <- (stratV-1)*Step[treat] + Low[treat]
    receiv <- trans * Rate[treat]
    pay1 <- 1 + (DictEndow[treat]-trans) / ifelse (Moderate==2, 10, 1)
    pay2 <- 1 + (RecEndow[treat]+receiv) / ifelse (Moderate==2, 10, 1)
    
    if (abs(alpha[2])<1e-10) {

        u <- pay1^(1-alpha[1]) * pay2^alpha[1]

    } else {

        s <- (1-alpha[1]) * pay1^alpha[2] + alpha[1] * pay2^alpha[2]
#  cat (pay, " => ", (1-alpha[1]) * pay[1]^alpha[2], "+", alpha[1] * pay[2]^alpha[2], "=", s, "\n")

        s[is.na(s)] <- -1e3
        FL <- (s>0)
        u <- s
        u[FL] <- s[FL]^(1/alpha[2])
        u[!FL] <- -pmax(0.001,abs(s[!FL]))^(1/alpha[2])
    }

    return (u)
}


ConvertToHighPrec <- function (x) {  #Input: log-scale
  y <- rep (0, 2)

#  cat (x)
  
  if (x> 1.0e16) y <- c(1,x / log(10))
  else if (x< -1.0e12) y <- c(0,0)
  else {
    y[2] <- round ( x / log(10) )
    y[1] <- exp( x - y[2]*log(10))
  }
  
#  cat (x, " => ", y, "\n")
  
  return (y)
}

ConvertToLogScale <- function (y) { #Input: High precision

  x <- max (-1e13, y[2] * log(10) + log(y[1]))
  
  return (x)
}

ConvertToRealScale <- function (y) { #Input: High precision

  x <- y[1] * 10^y[2]
  
  return (x)
}

AddHighPrec <- function (x1, x2) {
  y <- rep (0, 2)

  if (x1[1]==0) return (x2)
  if (x2[1]==0) return (x1)

  if (x1[2]>=x2[2]) {
    y[1] <- x1[1] + x2[1] * 10^(x2[2]-x1[2])
    y[2] <- x1[2]
  } else {
    y[1] <- x2[1] + x1[1] * 10^(x1[2]-x2[2])
    y[2] <- x2[2]
  }
  
  return (y)
}

DivideHighPrec <- function (x1, x2) {
  y <- rep (0, 2)

  y[2] <- x1[2] - x2[2]
  y[1] <- x1[1] / x2[1]
  
  return (y)
}

MultiplyHighPrec <- function (x1, x2) {
  y <- rep (0, 2)

  y[2] <- x1[2] + x2[2]
  y[1] <- x1[1] * x2[1]
  
  return (y)
}


#######################################################################################
#######################################################################################
#######################################################################################


NormalizeTypeShares <- function (pars) {
  shares <- pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]

  if (AnzTypes > 1) {
    if (sum(shares[2:length(shares)]) > 1) {
      shares[1] <- 0
      shares <- shares/sum(shares)
    } else {
      shares[1] <- 1 - sum(shares[2:length(shares)])
    }
  } else {
    shares <- 1
  }
  
  pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)] <- shares 

  return (pars)
}

GetIndivTypeStrat <- function (pars, t, subj, treat) {
    
#  share <- pars[(t-1)*(AnzTypePars+1) + 1]
  TypePars <- pars[(t-1)*(AnzTypePars+1) + 1 + 1:AnzTypePars]
  
#  if (TypePars[1]>=0) cat ("TypePars =", TypePars, "\n")
  TypePars[1] <- pmin (1e6, pmax(0, TypePars[1]))
  strat <- pmax(1e-36, GetTypeStrat (TypePars, subj=subj, treat=treat))

  if (is.na(sum(strat)) || sum(strat)>1.01) {
      lastPars <<- pars
      cat ("\n\nXXX Pars:", pars, "\t subj", subj, "\t treat", treat, "\n  =>  Sum =", sum(strat), "  (", strat, ")\n\n")
  }

  return (strat)
}

if (!exists("MixMode")) MixMode <- 0
lastStratMoms <- array (0, c(50, 10))
lastStratLength <- rep (0, 50)
lastStratM <- 0

truncQnorm <- function (q, m, sd) {
    if (pnorm (-0.5, m=m, sd=sd)>1-1e-2) m <- -0.5
    if (pnorm (0.5, m=m, sd=sd)<1e-2) m <- 0.5
    
    if (sd>1e-3) q2 <- pnorm (-0.5, m=m, sd=sd) + q * (pnorm (0.5, m=m, sd=sd) - pnorm (-0.5, m=m, sd=sd))
    else { q2 <- q*0+0.5 ; sd <- 1e-2 }

#    for (i in 1:length(q)) cat ("q", q[i], "-> q2", q2[i], "\t m =", m, "\tsd =", sd, "\t=>", qnorm (q2[i], m=m, sd=sd), "\n")
    
    return (qnorm (q2, m=m, sd=sd))
}
    
GetLL <- function (pars, RelTr=RelTreats) {
  pars <- pars * parFac
  if (length(pars)==length(FixedParsFL)) pars[FixedParsFL] <- FixedParsVals[FixedParsFL]
  
  ParMin <- rep(baseParMin, AnzTypes)
  ParMax <- rep(baseParMax, AnzTypes)

  penFac <- 1 + (sum(pmax (0, pars-ParMax)) - sum(pmin (0, pars-ParMin)))^2
  if (GetLLLog2)
    cat ("GetLL: ", " (", round(pars, digits=5), ")  =>  penFac =", penFac, "\n")

  pars <- pmin (ParMax, pmax(ParMin, pars))
  pars <- NormalizeTypeShares (pars)

  CallArgs <- paste(c(RelTr, AnzTypes, pars), collapse=" ")
  pos <- match (CallArgs, GetLLCallArgs)
  if (!is.na(pos)) {
    ret <- GetLLCallVals[pos]
    LL <- -ret/penFac
    if (GetLLLog2) cat (c("GetLL: ", " (", round(pars, digits=5), ")"))
    if (GetLLLog2) cat (c("  \t=>  LL =", round(LL, digits=5), "  ( ret =", round(ret, digits=5), ")\n"))

    return (ret)
  }
    
  if (GetLLLog) cat (c("GetLL (", GetLLCt, "): ", " (", paste (round(pars, digits=5), collapse=","), ")"))

  AnzSimul <- ifelse (MixMode %in% 2:5, 1024, 1024)
  IndivLL <- array (1, c(AnzSubj, AnzTypes, AnzSimul))
  StratM <- array (0, c(50, 1000))
  for (treat in RelTr) {
    for (type in 1:AnzTypes) {
        if (exists("MixTypesV") && (type %in% MixTypesV || type == AnzTypes)) {
            if (MixMode %in% 2:5) {
                pars2 <- pars
                d <- pars[c(2,5:9) + (type-1) * (AnzTypePars+1)]
                d2 <- pars[c(3:4) + (type-1) * (AnzTypePars+1)]
                for (s in 1:AnzSimul) {
                    if (MixMode==2) {
                        pars2[2 + (type-1)*(AnzTypePars+1)] <- pmax(0, qnorm (qrngMat[s,1], m=d[1], sd=abs(d[6])))
                        pars2[5 + (type-1)*(AnzTypePars+1)] <- truncQnorm (qrngMat[s,2], m=d[2], sd=abs(d[4]))
                        pars2[6 + (type-1)*(AnzTypePars+1)] <- qnorm (qrngMat[s,3], m=d[3], sd=abs(d[5]))
                        
                    } else if (MixMode==3) {
                        pars2[2 + (type-1)*(AnzTypePars+1)] <- pmax(0, exp(qnorm (qrngMat[s,1], m=d[1], sd=abs(d[6]))))
                        pars2[5 + (type-1)*(AnzTypePars+1)] <- truncQnorm (qrngMat[s,2], m=d[2], sd=abs(d[4]))
                        pars2[6 + (type-1)*(AnzTypePars+1)] <- qnorm (qrngMat[s,3], m=d[3], sd=abs(d[5]))
                        
                    } else if (MixMode==4) {
                        pars2[2 + (type-1)*(AnzTypePars+1)] <- pmax(0, exp(qnorm (qrngMat[s,1], m=d[1], sd=abs(d[6]))))
                        pars2[3 + (type-1)*(AnzTypePars+1)] <- pmax(0, exp(qnorm (qrngMat[s,4], m=d2[1], sd=abs(d2[2]))))
                        pars2[5 + (type-1)*(AnzTypePars+1)] <- truncQnorm (qrngMat[s,2], m=d[2], sd=abs(d[4]))
                        pars2[6 + (type-1)*(AnzTypePars+1)] <- qnorm (qrngMat[s,3], m=d[3], sd=abs(d[5]))
                        
                    } else if (MixMode==5) {
                        pars2[3 + (type-1)*(AnzTypePars+1)] <- pmax(0, exp(qnorm (qrngMat[s,1], m=d2[1], sd=abs(d2[2]))))
                        pars2[5 + (type-1)*(AnzTypePars+1)] <- truncQnorm (qrngMat[s,2], m=d[2], sd=abs(d[4]))
                        pars2[6 + (type-1)*(AnzTypePars+1)] <- qnorm (qrngMat[s,3], m=d[3], sd=abs(d[5]))
                    }

#                    print (pars) ; print (pars2)

                    if (pars2[2 + (type-1)*(AnzTypePars+1)]<0) cat ("pars2[type", type, "] =", pars2[(type-1)*(AnzTypePars+1) + 1 + 0:AnzTypePars], "\n")
                    
                    StratProbs <- GetIndivTypeStrat (pars2, type, subj=1, treat=treat)
                    lastStratLength[treat] <<- sl <- length(StratProbs)
                    StratM[treat,1:sl] <- StratM[treat,1:sl] + StratProbs/AnzSimul
                    for (subj in 1:AnzSubj) {
                        strat <- data[subj,treat]
                        NormStrat <- round((Up[treat] - Low[treat])/Step[treat]) - strat + 1
                        IndivLL[subj,type,s] <- IndivLL[subj,type,s] * StratProbs[NormStrat]
                    }
                }
            } else {
                pars2 <- pars
                d <- pars[5:8 + (type-1) * (AnzTypePars+1)]
                for (s in 1:AnzSimul) {
                    pars2[5 + (type-1)*(AnzTypePars+1)] <- truncQnorm (qrngMat[s,1], m=d[1], sd=abs(d[3]))
                    pars2[6 + (type-1)*(AnzTypePars+1)] <- qnorm (qrngMat[s,2], m=d[2], sd=abs(d[4]))
                    
                    StratProbs <- GetIndivTypeStrat (pars2, type, subj=1, treat=treat)
                    lastStratLength[treat] <<- sl <- length(StratProbs)
                    StratM[treat,1:sl] <- StratM[treat,1:sl] + StratProbs/AnzSimul
                    for (subj in 1:AnzSubj) {
                        strat <- data[subj,treat]
                        NormStrat <- round((Up[treat] - Low[treat])/Step[treat]) - strat + 1
                        IndivLL[subj,type,s] <- IndivLL[subj,type,s] * StratProbs[NormStrat]
                    }
                }
            }
        } else {
            if (pars[2 + (type-1)*(AnzTypePars+1)]<0) {
                pars[2 + (type-1)*(AnzTypePars+1)] <- max(0, pars[2 + (type-1)*(AnzTypePars+1)])
  #            cat ("pars[type", type, "] =", pars[(type-1)*(AnzTypePars+1) + 1 + 0:AnzTypePars], "\n")
            }
                    
            StratProbs <- GetIndivTypeStrat (pars, type, subj=1, treat=treat)
            lastStratLength[treat] <<- sl <- length(StratProbs)
            StratM[treat,1:sl] <- StratM[treat,1:sl] + StratProbs
            for (subj in 1:AnzSubj) {
                strat <- data[subj,treat]
                NormStrat <- round((Up[treat] - Low[treat])/Step[treat]) - strat + 1
                IndivLL[subj,type,] <- IndivLL[subj,type,] * StratProbs[NormStrat]
            }
        }
    }
  }

  lastStratM <<- StratM
  
  for (treat in RelTr) {
      AnzStrats <- lastStratLength[treat]
      m <- calcMoms (treat, probV=StratM[treat, 1:AnzStrats])
      lastStratMoms[treat, 1:length(m)] <<- m
  }

  IndivLL2 <- array (1, c(AnzSubj, AnzTypes))
  for (type in 1:AnzTypes)
      for (subj in 1:AnzSubj)
          IndivLL2[subj,type] <- mean (IndivLL[subj,type,])
  
  LL <- 0
  lastIndivLL <<- rep (0, AnzSubj)
  shares <- pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]
  lastPostClass <<- array (0, c(AnzSubj,AnzTypes))
  for (subj in 1:AnzSubj) {
    LL <- LL + log (max (1e-300, sum (shares * IndivLL2[subj,])))
    lastIndivLL[subj] <<- log (sum (shares * IndivLL2[subj,]))
    lastPostClass[subj,] <<- shares * IndivLL2[subj,] / sum (shares * IndivLL2[subj,])
  }
  
  ret <- -LL*penFac
  if (GetLLLog || GetLLLog2)
    cat (c("  \t=>  LL =", round(LL, digits=5), "  ( ret =", round(ret, digits=5), ")\n"))
  else if (0 && GetLLLog) cat (round(LL, digits=3), "\n")

  GetLLCt <<- GetLLCt + 1
  
  if (FLSaveRes) {
    GetLLCallArgs <<- c(GetLLCallArgs, CallArgs)                  
    GetLLCallVals <<- c(GetLLCallVals, ret)
    save (GetLLCallArgs, GetLLCallVals, file = DataFile)
  }

  return (ret)
}

GetLL2 <- function (pars, GetUtil=UAltr, RelTr=RelTreats) { # nutzt lastPostClass[subj,] als weights
  pars <- pars * parFac
  if (length(pars)==length(FixedParsFL)) pars[FixedParsFL] <- FixedParsVals[FixedParsFL]
  
  ParMin <- rep(baseParMin, AnzTypes)
  ParMax <- rep(baseParMax, AnzTypes)

  pars <- pmin (ParMax, pmax(ParMin, pars))
  pars <- NormalizeTypeShares (pars)

  AnzSimul <- ifelse (MixMode %in% 2:5, 1024, 1024)
  IndivLL <- array (1, c(AnzSubj, AnzTypes, AnzSimul))
  for (treat in RelTr) {
    for (type in 1:AnzTypes) {
        if (exists("MixTypesV") && (type %in% MixTypesV || type == AnzTypes)) {
            if (MixMode %in% 2:5) {
                pars2 <- pars
                d <- pars[c(2,5:9) + (type-1) * (AnzTypePars+1)]
                d2 <- pars[c(3:4) + (type-1) * (AnzTypePars+1)]
                for (s in 1:AnzSimul) {
                    if (MixMode==2) {
                        pars2[2 + (type-1)*(AnzTypePars+1)] <- pmax(0, qnorm (qrngMat[s,1], m=d[1], sd=abs(d[6])))
                        pars2[5 + (type-1)*(AnzTypePars+1)] <- truncQnorm (qrngMat[s,2], m=d[2], sd=abs(d[4]))
                        pars2[6 + (type-1)*(AnzTypePars+1)] <- qnorm (qrngMat[s,3], m=d[3], sd=abs(d[5]))
                        
                    } else if (MixMode==3) {
                        pars2[2 + (type-1)*(AnzTypePars+1)] <- pmax(0, exp(qnorm (qrngMat[s,1], m=d[1], sd=abs(d[6]))))
                        pars2[5 + (type-1)*(AnzTypePars+1)] <- truncQnorm (qrngMat[s,2], m=d[2], sd=abs(d[4]))
                        pars2[6 + (type-1)*(AnzTypePars+1)] <- qnorm (qrngMat[s,3], m=d[3], sd=abs(d[5]))
                        
                    } else if (MixMode==4) {
                        pars2[2 + (type-1)*(AnzTypePars+1)] <- pmax(0, exp(qnorm (qrngMat[s,1], m=d[1], sd=abs(d[6]))))
                        pars2[3 + (type-1)*(AnzTypePars+1)] <- pmax(0, exp(qnorm (qrngMat[s,4], m=d2[1], sd=abs(d2[2]))))
                        pars2[5 + (type-1)*(AnzTypePars+1)] <- truncQnorm (qrngMat[s,2], m=d[2], sd=abs(d[4]))
                        pars2[6 + (type-1)*(AnzTypePars+1)] <- qnorm (qrngMat[s,3], m=d[3], sd=abs(d[5]))
                        
                    } else if (MixMode==5) {
                        pars2[3 + (type-1)*(AnzTypePars+1)] <- pmax(0, exp(qnorm (qrngMat[s,1], m=d2[1], sd=abs(d2[2]))))
                        pars2[5 + (type-1)*(AnzTypePars+1)] <- truncQnorm (qrngMat[s,2], m=d[2], sd=abs(d[4]))
                        pars2[6 + (type-1)*(AnzTypePars+1)] <- qnorm (qrngMat[s,3], m=d[3], sd=abs(d[5]))
                    }
                    
                    StratProbs <- GetIndivTypeStrat (pars2, type, subj=1, treat=treat)
                    for (subj in 1:AnzSubj) {
                        strat <- data[subj,treat]
                        NormStrat <- round((Up[treat] - Low[treat])/Step[treat]) - strat + 1
                        IndivLL[subj,type,s] <- IndivLL[subj,type,s] * StratProbs[NormStrat]
                    }
                }
            } else {
                pars2 <- pars
                d <- pars[5:8 + (type-1) * (AnzTypePars+1)]
                for (s in 1:AnzSimul) {
                    pars2[5 + (type-1)*(AnzTypePars+1)] <- truncQnorm (qrngMat[s,1], m=d[1], sd=abs(d[3]))
                    pars2[6 + (type-1)*(AnzTypePars+1)] <- qnorm (qrngMat[s,2], m=d[2], sd=abs(d[4]))

                    StratProbs <- GetIndivTypeStrat (pars2, type, subj=1, treat=treat)
                    for (subj in 1:AnzSubj) {
                        strat <- data[subj,treat]
                        NormStrat <- round((Up[treat] - Low[treat])/Step[treat]) - strat + 1
                        IndivLL[subj,type,s] <- IndivLL[subj,type,s] * StratProbs[NormStrat]
                    }
                }
            }
        } else {
            pars[2 + (type-1)*(AnzTypePars+1)] <- max(0, pars[2 + (type-1)*(AnzTypePars+1)])
            StratProbs <- GetIndivTypeStrat (pars, type, subj=1, treat=treat)
            for (subj in 1:AnzSubj) {
                strat <- data[subj,treat]
                NormStrat <- round((Up[treat] - Low[treat])/Step[treat]) - strat + 1
                IndivLL[subj,type,] <- IndivLL[subj,type,] * StratProbs[NormStrat]
            }
        }
    }
  }
  
  IndivLL2 <- array (1, c(AnzSubj, AnzTypes))
  LL <- 0
  shares <- pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]
  for (subj in 1:AnzSubj) {
      for (type in 1:AnzTypes) IndivLL2[subj,type] <- mean (IndivLL[subj,type,])
      iLL <- lastPostClass[subj,] * IndivLL2[subj,]
      LL <- LL + log (sum (iLL))
      lastIndivLL[subj] <<- log (sum (iLL))
      lastPostClass[subj,] <<- iLL / sum (iLL)
  }
  
  ret <- -LL

  return (ret)
}

library (entropy)
empiricalEntropy <- FALSE

calcMoms <- function (treat, probV) {
    stratsV <- 0:((Up[treat] - Low[treat]) / Step[treat])
    
    sMean <- sum (stratsV * probV)
    
    sVar <- sum (stratsV^2 * probV) - sMean^2
    
    pv2 <- probV[probV!=0]

    if (empiricalEntropy) sEntr <- entropy (round(probV*AnzSubj), method="MM")
    else sEntr <- -sum (pv2 * log(pv2))
    
    sExpEntr <- exp(sEntr)
    
    sh0 <- probV[1]
    shPos <- 1-probV[1]

    # Calc Equal Split
    # DictEndow[treat]- (strat-1)*Step[treat] - Low[treat] = RecEndow[treat] + (strat-1)*Step[treat]*Rate[treat] + Low[treat]*Rate[treat]
    # (1+Rate[treat]) * (strat-1)*Step[treat] = DictEndow[treat] - RecEndow[treat] - (1+Rate[treat]) * Low[treat]
      
    es <- (DictEndow[treat] - RecEndow[treat] - (1+Rate[treat]) * Low[treat]) / ((1+Rate[treat]) * Step[treat])
    sh50 <- probV[1+round(es)]

    cs <- cumsum(-sort (-pv2))
    min90 <- which.max(cs>=0.9)

    return (c(sMean, sVar, sEntr, sh0, sh50, sExpEntr, min90, shPos))
}


calcLLmax <- function (RelTr) {

  AnzClasses <- 4
    
  indivZeroes <- array (0, c(AnzSubj))
  for (subj in 1:AnzSubj) indivZeroes[subj] <- sum(Up/Step - data[subj,]==0)
  indivClass <- 1 + pmin (indivZeroes, AnzClasses-1)
  shares <- rep (0, AnzClasses)
  for (cla in 1:AnzClasses) shares[cla] <- sum(indivClass==cla)/length(indivClass)

  lastIndivClass <<- indivClass
  lastShares <<- shares

  if (0) {
      numMax0 <- sum (indivMax==0) / length(indivMax)
      setMax0 <- (1:AnzSubj)[indivMax==0]
      invSetMax0 <- (1:AnzSubj)[indivMax!=0]  
      lastIndivMax <<- indivMax
      lastSetMax0 <<- setMax0
  }

  IndivLL <- array (0, c(AnzSubj))
  for (treat in RelTr) {
      lastStratLength[treat] <<- (Up[treat] - Low[treat]) / Step[treat] + 1

      empStrat <- rep (0, lastStratLength[treat])
      empStratM <- array (0, c(AnzClasses, lastStratLength[treat]))
      for (cla in 1:AnzClasses) {
          subjInd <- (indivClass==cla)
          
          for (keep in unique(data[,treat])) {
              trans <- lastStratLength[treat] - keep
              if (cla==1) empStrat[trans] <- sum(keep == data[,treat]) / length(data[,treat])
              
              empStratM[cla,trans] <- sum(keep == data[subjInd,treat]) / sum(subjInd)
          }
      }

      m <- calcMoms (treat, probV=empStrat)
      lastStratMoms[treat, 1:length(m)] <<- m
    
      for (subj in 1:AnzSubj) {
          cla <- indivClass[subj]
          if (treat==RelTr[1]) IndivLL[subj] <- log (shares[cla])

          trans <- lastStratLength[treat] - data[subj,treat]
          IndivLL[subj] <- IndivLL[subj] + log (empStratM[cla, trans])

#          cat ("Subj", subj, "=>", empStratM[cla, trans], IndivLL[subj], "\n")
      }
  }

  if (0 && numMax0>0)  {
      IndivLL[indivMax==0] <- log (numMax0)
      IndivLL[indivMax!=0] <- log (1-numMax0) + IndivLL[indivMax!=0]
  }

  lastIndivLL <<- IndivLL
  return (sum(IndivLL))
}

calcLLmin <- function (RelTr) {

  IndivLL <- array (0, c(AnzSubj))
  for (treat in RelTr) {
      lastStratLength[treat] <<- (Up[treat] - Low[treat]) / Step[treat] + 1
      
      m <- calcMoms (treat, probV=rep (1/lastStratLength[treat], lastStratLength[treat]))
      lastStratMoms[treat, 1:length(m)] <<- m
    
      for (subj in 1:AnzSubj) {
          stratPr <- 1 / lastStratLength[treat]
          IndivLL[subj] <- IndivLL[subj] + log (stratPr)
      }
  }

  lastIndivLL <<- IndivLL
  return (sum(IndivLL))
}


source ("../print-input.R")
