
if (!exists("Moderate")) Moderate <- 0


GetTypeStratHet <- function (pars, subj=1, treat, GetUtil=UAltr) { 

#  cat ("GetTypeStrat (", round(pars, digits=5), ")\n")

  if (exists("Up") && exists("Low")) {
      AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
      lineNr <- subj
  } else {
    lineNr <- (1:length(RefData$Up))[RefData$Treat==treat & RefData$ID==subj]
    AnzStrats <- RefData$Up[lineNr] + 1
    prLevs <- GetProminenceLevs (treat, subj=lineNr)
  }

  prLevs <- GetProminenceLevs (treat, subj=subj)
  
  lambda <- pars[1] * pars[2]^prLevs
#  rho <- pars[2]
#  sigma <- pars[3]
  alpha <- pars[4:5]

  if (exists ("UCESAltrV")) utils <- UCESAltrV (stratV=1:AnzStrats, alpha, subj=lineNr, treat)
  else {
      utils <- rep (0, AnzStrats)
      for (strat in 1:AnzStrats)
          utils[strat] <- UCESAltr(strat, alpha, subj=lineNr, treat)
  }

  if (fnameP1=="list" && treat==2) utils[2] <- -1e3

  utils[is.na(utils)] <- 1e6
  utils <- pmax(-1e6, pmin(1e6, utils))
  if (Moderate==1) utils <- utils / pmax(1e-3, max(utils) - min(utils))
  if (Moderate==5) utils <- utils / pmax(1e-3, max(abs(utils)))
    
#  utils <- lambda * (utils - max (utils))
  props <- exp(lambda * utils - max (lambda * utils))
  P <- props/sum(props)

  if (length(P)==0 || sum(is.na(P))) {
    cat ("\nXXX - Problem!!\n")
    cat ("utils:", round(utils, digits=2), "\n")
    cat ("lambda:", round(lambda, digits=2), "\n")
    cat ("props:", round(props, digits=2), "\n")
    cat ("P:", round(P, digits=2), "\n")
  }
  
  if (treat==0) cat ("P :", P, " => Sum =", sum(P), "\n")

  return (P)
}



GetTypeStratProm <- function (pars, subj=1, treat, GetUtil=UAltr) { 

#  cat ("GetTypeStrat (", round(pars, digits=5), ")\n")

  if (exists("Up") && exists("Low")) {
      AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
      prLevs <- GetProminenceLevs (treat, subj=subj, FixPromZero=2)
      lineNr <- subj
  } else {
    lineNr <- (1:length(RefData$Up))[RefData$Treat==treat & RefData$ID==subj]
    AnzStrats <- RefData$Up[lineNr] + 1
    prLevs <- GetProminenceLevs (treat, subj=lineNr, FixPromZero=2)
  }
  
  lambda <- pars[1]
  rho <- pars[2]
#  sigma <- pars[3]
  alpha <- pars[4:5]
  
#  cat ("prLevs:", prLevs, "\n")
#  cat ("lambda:", round(lambda, digits=2), "\n")
  
  if (exists ("UCESAltrV")) utils <- UCESAltrV (stratV=1:AnzStrats, alpha, subj=lineNr, treat)
  else {
      utils <- rep (0, AnzStrats)
      for (strat in 1:AnzStrats)
          utils[strat] <- UCESAltr(strat, alpha, subj=lineNr, treat)
  }

  if (fnameP1=="list" && treat==2) utils[2] <- -1e3

  utils[is.na(utils)] <- 1e6
  utils <- pmax(-1e6, pmin(1e6, utils))
  if (Moderate==1) utils <- utils / pmax(1e-3, max(utils) - min(utils))
  if (Moderate==5) utils <- utils / pmax(1e-3, max(abs(utils)))

  utils <- lambda * utils + rho * prLevs
  props <- exp(utils - max (utils))
  P <- props / sum(props)
  if (treat==0) cat ("P :", P, " => Sum =", sum(P), "\n")

  return (P)
}


GetTypeStratPALM <- function (pars, subj=1, treat, GetUtil=UAltr) { 

#  cat ("GetTypeStrat (", round(pars, digits=5), ")\n")

  if (exists("Up") && exists("Low")) {
    AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
    prLevs <- GetProminenceLevs (treat, subj=subj, FixPromZero=2)
    lineNr <- subj
    
  } else { # for Capp
    lineNr <- (1:length(RefData$Up))[RefData$Treat==treat & RefData$ID==subj]
    AnzStrats <- RefData$Up[lineNr] + 1
    prLevs <- GetProminenceLevs (treat, subj=lineNr, FixPromZero=2)

#    cat ("subj, treat, lineNr =>", subj, treat, lineNr, "=> AnzStrats", AnzStrats, "\n")
  }
  
  lambda <- pars[1]
  rho <- pars[2]
#  sigma <- pars[3]
  alpha <- pars[4:5]
  
#  cat ("prLevs:", prLevs, "\n")
#  cat ("lambda,rho:", round(c(lambda, rho), digits=2), "\n")
  
  if (exists ("UCESAltrV")) utils <- UCESAltrV (stratV=1:AnzStrats, alpha, subj=lineNr, treat)
  else {
      utils <- rep (0, AnzStrats)
      for (strat in 1:AnzStrats)
          utils[strat] <- UCESAltr(strat, alpha, subj=lineNr, treat)
  }

  if (fnameP1=="list" && treat==2) utils[2] <- -1e3

#  cat ("utils:", utils, "\n")
  utils[is.na(utils)] <- 1e6
  utils <- pmax(-1e6, pmin(1e6, utils))
  if (Moderate==1) utils <- utils / pmax(1e-3, max(utils) - min(utils))
  if (Moderate==5) utils <- utils / pmax(1e-3, max(abs(utils)))
    
  utils <- lambda * utils
  utils <- exp(utils - max (utils))

  mu <- utils / sum (utils)      # these are the logit choice probs

  levV <- - sort ( -unique (prLevs) )
  prevLevProd <- 1

  P <- utils*0

  lastPrLevs <<- prLevs
  lastUtils <<- utils
    
  for (l in levV) {
      thisLev <- (1:length(prLevs))[prLevs==l]
      
      P[thisLev] <- mu[thisLev] * prevLevProd
      prevLevProd <- prevLevProd * pmax (0,  1 - rho * sum(mu[thisLev]) )   # note the difference to palm3: pmax (0, ...) and unrestricted rho
      
#      cat ("P =", P[thisLev], "=>", prevLevProd, sum(probsThisLev), "\n")
  }
    

  if (treat==0) cat ("P :", P, " => Sum =", sum(P), "\n")

  P <- P / sum(P)

  return (P)
}


GetTypeStratPALM2 <- function (pars, subj=1, treat, GetUtil=UAltr) { 

#  cat ("GetTypeStrat (", round(pars, digits=5), ")\n")

  if (exists("Up") && exists("Low")) {
    AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
    prLevs <- GetProminenceLevs (treat, subj=subj, FixPromZero=2)
    lineNr <- subj
    
  } else { # for Capp
    lineNr <- (1:length(RefData$Up))[RefData$Treat==treat & RefData$ID==subj]
    AnzStrats <- RefData$Up[lineNr] + 1
    prLevs <- GetProminenceLevs (treat, subj=lineNr, FixPromZero=2)

#    cat ("subj, treat, lineNr =>", subj, treat, lineNr, "=> AnzStrats", AnzStrats, "\n")
  }
  
  lambda <- pars[1]
  rho <- pars[2]
#  sigma <- pars[3]
  alpha <- pars[4:5]
  
#  cat ("prLevs:", prLevs, "\n")
#  cat ("lambda,rho:", round(c(lambda, rho), digits=2), "\n")
  
  if (exists ("UCESAltrV")) utils <- UCESAltrV (stratV=1:AnzStrats, alpha, subj=lineNr, treat)
  else {
      utils <- rep (0, AnzStrats)
      for (strat in 1:AnzStrats)
          utils[strat] <- UCESAltr(strat, alpha, subj=lineNr, treat)
  }

  if (fnameP1=="list" && treat==2) utils[2] <- -1e3

#  cat ("utils:", utils, "\n")
  utils[is.na(utils)] <- 1e6
  utils <- pmax(-1e6, pmin(1e6, utils))
  if (Moderate==1) utils <- utils / pmax(1e-3, max(utils) - min(utils))
  if (Moderate==5) utils <- utils / pmax(1e-3, max(abs(utils)))
    
  utils <- lambda * utils
  utils <- exp(utils - max (utils))

  mu <- utils / sum (utils)      # these are the logit choice probs

  levV <- - sort ( -unique (prLevs) )
  prevLevProd <- 1

  P <- utils*0

  lastPrLevs <<- prLevs
  lastUtils <<- utils
    
  for (l in levV) {
      thisLev <- (1:length(prLevs))[prLevs==l]
      
      P[thisLev] <- mu[thisLev] * prevLevProd
      prevLevProd <- prevLevProd * pmax(1e-6,1 - sum(mu[thisLev]))^rho   # note the difference to palm: rho is exponent, not factor 
      
#      cat ("P =", P[thisLev], "=>", prevLevProd, sum(mu[thisLev]), "\n")
  }
    

  if (treat==0) cat ("P :", P, " => Sum =", sum(P), "\n")

  P <- P / sum(P)

  return (P)
}


GetTypeStratPALM3 <- function (pars, subj=1, treat, GetUtil=UAltr) { 

#  cat ("GetTypeStrat (", round(pars, digits=5), ")\n")

  if (exists("Up") && exists("Low")) {
    AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
    prLevs <- GetProminenceLevs (treat, subj=subj, FixPromZero=2)
    lineNr <- subj
    
  } else { # for Capp
    lineNr <- (1:length(RefData$Up))[RefData$Treat==treat & RefData$ID==subj]
    AnzStrats <- RefData$Up[lineNr] + 1
    prLevs <- GetProminenceLevs (treat, subj=lineNr, FixPromZero=2)

#    cat ("subj, treat, lineNr =>", subj, treat, lineNr, "=> AnzStrats", AnzStrats, "\n")
  }
  
  lambda <- pars[1]
  rho <- pmin (1, pars[2])
#  sigma <- pars[3]
  alpha <- pars[4:5]
  
#  cat ("prLevs:", prLevs, "\n")
#  cat ("lambda,rho:", round(c(lambda, rho), digits=2), "\n")
  
  if (exists ("UCESAltrV")) utils <- UCESAltrV (stratV=1:AnzStrats, alpha, subj=lineNr, treat)
  else {
      utils <- rep (0, AnzStrats)
      for (strat in 1:AnzStrats)
          utils[strat] <- UCESAltr(strat, alpha, subj=lineNr, treat)
  }

  if (fnameP1=="list" && treat==2) utils[2] <- -1e3

#  cat ("utils:", utils, "\n")
  utils[is.na(utils)] <- 1e6
  utils <- pmax(-1e6, pmin(1e6, utils))
  if (Moderate==1) utils <- utils / pmax(1e-3, max(utils) - min(utils))
  if (Moderate==5) utils <- utils / pmax(1e-3, max(abs(utils)))
    
  utils <- lambda * utils
  utils <- exp(utils - max (utils))

  mu <- utils / sum (utils)      # these are the logit choice probs

  levV <- - sort ( -unique (prLevs) )
  prevLevProd <- 1

  P <- utils*0

  lastPrLevs <<- prLevs
  lastUtils <<- utils
    
  for (l in levV) {
      thisLev <- (1:length(prLevs))[prLevs==l]
      
      P[thisLev] <- mu[thisLev] * prevLevProd
      prevLevProd <- prevLevProd * ( 1 - rho * sum(mu[thisLev]) )   # note the difference to palm 2: multiplicative and based on mu
      
#      cat ("P =", P[thisLev], "=>", prevLevProd, sum(probsThisLev), "\n")
  }
    

  if (treat==0) cat ("P :", P, " => Sum =", sum(P), "\n")

  P <- P / sum(P)

  return (P)
}



GetTypeStratOleHighP <- function (pars, subj=1, treat, GetUtil=UAltr) {    # Uses high precision arithmetics, called by default

#  cat ("GetTypeStrat (", round(pars, digits=5), ")\n")

  if (exists("Up") && exists("Low")) {
      AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
      lineNr <- subj
  } else {
    lineNr <- (1:length(RefData$Up))[RefData$Treat==treat & RefData$ID==subj]
    AnzStrats <- RefData$Up[lineNr] + 1
  }
  M <- 2 * ceiling (AnzStrats/4)

  lambda <- pars[1]
  rho <- pmin (1, pars[2])
  if (exists("FLSimp") && FLSimp) sigma <- 1 else sigma <- pars[3]
  alpha <- pars[4:5]
  w <- dnorm (0:M - M/2, sd=sigma, log=TRUE)
  w <- w  - log(sum(exp(w)))
  
  utils <- rep (0, AnzStrats)
  for (strat in 1:AnzStrats)
    utils[strat] <- UCESAltr(strat, alpha, subj=lineNr, treat)

  if (fnameP1=="list" && treat==2) utils[2] <- -1e3

  utils[is.na(utils)] <- 1e6
  utils <- pmax(-1e6, pmin(1e6, utils))
  if (Moderate==1) utils <- utils / pmax(1e-3, max(utils) - min(utils))
  if (Moderate==5) utils <- utils / pmax(1e-3, max(abs(utils)))
  utils <- lambda * (utils - max (utils))
  
  J <- AnzStrats
  I <- rep (0, J+M)

  for (r in 1:(J+M)) {
    sumI <- rep (0, 2)
    for (j in (r-M):r) {
      if (j<1 || j>J) next
      u <- utils[j]/rho + w[r-j+1]
      uH <- ConvertToHighPrec (u)
#      cat (r, j, ":", sumI, "+", uH, "(", u, ")", "\n")
      sumI <- AddHighPrec (sumI, uH)
    }
    I[r] <- ConvertToLogScale (sumI)
    if (I[r]< -1e12 && r>=3) I[r] <- 2*I[r-1] - I[r-2]
  }
  if (I[2]< -1e12) I[2] <- 2*I[3] - I[4]
  if (I[1]< -1e12) I[1] <- 2*I[2] - I[3]
  I <- pmax(-1e4, I)
  if (treat==0 || is.infinite(sum(I))) cat ("I :", I, "\n")

  Qdenom <- rep (0,2)
  for (s in 1:(J+M)) 
    Qdenom <- AddHighPrec (Qdenom, ConvertToHighPrec (rho * I[s]))
  
  Q <- rep (0, J+M)
  for (r in 1:(J+M)) {
    x <- DivideHighPrec (ConvertToHighPrec (rho * I[r]), Qdenom)
    Q[r] <- ConvertToRealScale (x)
  }
  if (treat==0 || is.infinite(sum(Q))) cat ("Q :", Q, "\n")

  P <- rep (0, J)
  for (k in 1:J) {
    for (r in k:(k+M)) {
      q <- exp(utils[k]/rho + w[r-k+1] - I[r])
      P[k] <- P[k] + q * Q[r]
      if (is.nan(P[k])) print (c(k, r, q, utils[k]/rho + w[r-k+1] - I[r], Q[r], I[1:5]))
    }
  }
  if (treat==0) cat ("P :", P, " => Sum =", sum(P), "\n")

  return (P)
}



GetTypeStratOle <- function (pars, subj=1, treat, GetUtil=UAltr) {    # Regular ordered GEV, manages without high precision

#  cat ("GetTypeStrat (", round(pars, digits=5), ")\n")

  if (exists("Up") && exists("Low")) {
      AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
      lineNr <- subj
  } else {
    lineNr <- (1:length(RefData$Up))[RefData$Treat==treat & RefData$ID==subj]
    AnzStrats <- RefData$Up[lineNr] + 1
  }

  lambda <- pars[1]
  rho <- pmin (1, pars[2])
  if (exists("FLSimp") && FLSimp) sigma <- 1 else sigma <- pars[3]
  alpha <- pars[4:5]
    
#  M <- 2 * ceiling (AnzStrats/4)
  M <- 2 * ceiling (pmin (AnzStrats/4, qnorm (0.999, sd=sigma)))

  w <- dnorm (0:M - M/2, sd=sigma, log=TRUE)
  w <- w  - log(sum(exp(w)))
  
  if (exists ("UCESAltrV")) utils <- UCESAltrV (stratV=1:AnzStrats, alpha, subj=lineNr, treat)
  else {
      utils <- rep (0, AnzStrats)
      for (strat in 1:AnzStrats)
          utils[strat] <- UCESAltr(strat, alpha, subj=lineNr, treat)
  }

  if (fnameP1=="list" && treat==2) utils[2] <- -1e3

  utils[is.na(utils)] <- 1e6
  utils <- pmax(-1e6, pmin(1e6, utils))
  if (Moderate==1) utils <- utils / pmax(1e-3, max(utils) - min(utils))
  if (Moderate==5) utils <- utils / pmax(1e-3, max(abs(utils)))
  utils <- lambda * (utils - max (utils))
  
  J <- AnzStrats
  I <- rep (0, J+M)

  for (r in 1:(J+M)) {

    indV <- (r-M):r ; indV <- indV[indV>=1 & indV<=J]

#    b <- rep(0, max(indV)) ; for (j in indV) b[j] <- utils[j]/rho + w[r-j+1]
    b <- utils[indV]/rho + w[r-indV+1]

    maxI <- max(b)    
    sumI <- sum ( exp(b - maxI) )
    I[r] <- log(sumI) + maxI
  }

  maxQ <- max(rho * I)
  props <- exp(rho * I - maxQ)
  Q <- props / sum(props)

  P <- rep (0, J)
  for (k in 1:J) {
      rV <- k:(k+M)
      P[k] <- sum (exp (utils[k]/rho + w[rV-k+1] - I[rV]) * Q[rV])
      
#      for (r in k:(k+M)) {
#          q <- exp (utils[k]/rho + w[r-k+1] - I[r])
#          P[k] <- P[k] + q * Q[r]
#      }
  }
  if (treat==0) cat ("P :", P, " => Sum =", sum(P), "\n")

  return (P)
}



GetTypeStratLogit <- function (pars, subj=1, treat, GetUtil=UAltr) { # Neglects high precision arithmetics --> sufficiently precise with mult.\ logit

#  cat ("GetTypeStrat (", round(pars, digits=5), ")\n")

  if (exists("Up") && exists("Low")) {
      AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
      lineNr <- subj
  } else {
    lineNr <- (1:length(RefData$Up))[RefData$Treat==treat & RefData$ID==subj]
    AnzStrats <- RefData$Up[lineNr] + 1
#    cat ("subj:", subj, "\ttreat:", treat, "\tlineNr:", lineNr, "\tAnzStrats:", AnzStrats, "\n")
  }

  lambda <- pars[1]
#  rho <- pars[2]
#  sigma <- pars[3]
  alpha <- pars[4:5]
  
  if (exists ("UCESAltrV")) utils <- UCESAltrV (stratV=1:AnzStrats, alpha, subj=lineNr, treat)
  else {
      utils <- rep (0, AnzStrats)
      for (strat in 1:AnzStrats)
          utils[strat] <- UCESAltr(strat, alpha, subj=lineNr, treat)
  }

  if (fnameP1=="list" && treat==2) utils[2] <- -1e3

  utils[is.na(utils)] <- 1e6
  utils <- pmax(-1e6, pmin(1e6, utils))
  if (Moderate==1) utils <- utils / pmax(1e-3, max(utils) - min(utils))
  if (Moderate==5) utils <- utils / pmax(1e-3, max(abs(utils)))
  
  props <- exp(lambda * (utils - max (utils)))

  if (0) {
    lastUtils <<- utils
    lastLambda <<- lambda
    lastProps <<- props
  }
  
  P <- props / sum(props)
  if (treat==0) cat ("P :", P, " => Sum =", sum(P), "\n")

  return (P)
}
