

cat ("My pid:", Sys.getpid (), "\n\n")

source ("pics-garp-inp.R")

data <- c()
FlInSample <- TRUE


#######################################################################################
#######################################################################################
#######################################################################################


ExPostClass <- function (parsFunc, calcFunc, AnzTypes, filename="") {
  
  if (filename != "") pdf (filename, width=7, height=9.5)
  
  layout (rbind(c(1,2),c(3,4),c(5,6),c(7,8)),
#          heights=c(6.4,4.6),
#          widths=c(1.6,4,1.6,4.6,2.8,5.2),
#          respect=TRUE
          )

#  par (xaxs="r", yaxs="r", bty="n")
#  plot (c(0,10), c(0,1), type="n")
  
  par (xaxs="r", yaxs="r", bty="l", xpd=NA)
  for (treat in 1:AnzTreat) {
    pars <- parsFunc (AnzTypes, treat)
    shares <- pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]
    AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1

    StratProbs <- array (0, c(AnzTypes, AnzStrats))
    SumProbs <- array (0, c(AnzStrats))
    for (type in 1:AnzTypes) {
      StratProbs[type,] <- shares[type] * calcFunc (AnzTypes, type, treat)
      SumProbs <- SumProbs + StratProbs[type,] 
    }
    
    for (type in 1:AnzTypes) 
      StratProbs[type,] <- StratProbs[type,] / SumProbs
    
    strats <- (1:AnzStrats - 1) * Step[treat]

    plot (c(min(strats),max(strats)), c(0,1), xlab="Donation to recipient", ylab="Ex post probability", main=paste("Treatment", treat), type="n")
    
    for (type in 1:AnzTypes) 
      lines (strats, StratProbs[type,], lty=type)

    if (treat==1)
      legend (111,-0.35, paste("Type", 1:7), lty=1:7, cex=1, horiz=!TRUE, ncol=2)
  }

  if (filename != "") dev.off ()
}


#######################################################################################


GetBarProbs <- function (parsFunc, calcFunc, treat, nBars, AnzTypes) {
  pars <- parsFunc (AnzTypes, treat)
  shares <- pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]
  AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1

  AggProbs <- array (0, c(AnzStrats))
  for (type in 1:AnzTypes)  {
    StratProbs <- calcFunc (AnzTypes, type, treat)
    AggProbs <- AggProbs + shares[type] * StratProbs
  }

  barProbs <- rep (0, nBars)
  for (b in 1:nBars) {
    low <- ceiling ((AnzStrats-1) * (b-1) / nBars) + 1
    up <- ceiling ((AnzStrats-1) * b / nBars)
    barProbs[b] <- sum (AggProbs[low:up])
  }

  return (barProbs)
}

PlotBarMatrix <- function (treat) {
  RefData <- read.table (file="garp-dat.csv", header=T, sep=",")
  data <- RefData[,2:9]

  nBars <- 10
  AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
  strats <- (1:AnzStrats - 1) * Step[treat]
  
  relFreq <- rep (0, nBars)
  names <- rep ("", nBars)
  for (b in 1:nBars) {
    low <- AnzStrats * (nBars-b) / nBars
    up <- AnzStrats * (nBars-b+1.001) / nBars
    relFreq[b] <- sum(data[,treat]>=low & data[,treat]<up) / AnzSubj
    names[b] <- paste (AnzStrats-round(up), "-", AnzStrats-round(low)-1, sep="")
  }

#  cat ("Treat =", treat, "  =>  ", relFreq, "\n")

  barMatr <- array (0, c(7, nBars))
  barMatr[1,] <- GetBarProbs (parsGaussOLE, calcGaussOLE, treat, nBars, 7) - relFreq
  barMatr[2,] <- GetBarProbs (parsCoxCoeff, calcCoxCoeff, treat, nBars, 7) - relFreq
  barMatr[3,] <- GetBarProbs (parsNestCoeff, calcNestCoeff, treat, nBars, 7) - relFreq
#  barMatr[4,] <- GetBarProbs (parsNumNestCoeff, calcNumNestCoeff, treat, nBars, 7) - relFreq
  barMatr[4,] <- GetBarProbs (parsStructRegr, calcStructRegr, treat, nBars, 7) - relFreq
  barMatr[5,] <- GetBarProbs (parsStructBehav, calcStructBehav, treat, nBars, 7) - relFreq
  barMatr[6,] <- GetBarProbs (parsTobitRegr, calcTobitRegr, treat, nBars, 7) - relFreq
  barMatr[7,] <- GetBarProbs (parsTobit2Regr, calcTobit2Regr, treat, nBars, 7) - relFreq

  par (xaxs="r", yaxs="r", bty="l", xpd=NA)
  
  barplot (barMatr, beside=TRUE, names.arg=names, xlab="Tokens donated", ylab="Deviat. from rel. freq.", main=paste("Treatment", treat), ylim=c(-.2,.2))

  if (treat==1) {
    colors <- gray.colors(7)
    inset <- c(0.1, 0.1)

    legend (10, 0.53, c("Ordered GEV", "Rand Taste", "Nested Logit",
                        "Struct Regr", "Rand Behav", "Regression", "Extend. Regr."),
            fill=colors, inset=inset, cex=1.1, ncol=4)
  }

#  legend (111,-0.35, paste("Type", 1:7), lty=1:7, cex=1, horiz=!TRUE, ncol=2)
  
  return (barMatr)
}

BarMatrix <- function (filename="") {
  if (filename != "") pdf (filename, width=7, height=12)
  layout (rbind(c(9),c(1,2),c(3,4),c(5,6),c(7,8)))

  for (treat in 1:AnzTreat) 
    m <- PlotBarMatrix (treat)
  
  if (filename != "") dev.off ()
}

#######################################################################################


PlotBarMatrix2 <- function (treat) {
  RefData <- read.table (file="garp-dat.csv", header=T, sep=",")
  data <- RefData[,2:9]

  nBars <- 10
  AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
  strats <- (1:AnzStrats - 1) * Step[treat]
  
  relFreq <- rep (0, nBars)
  names <- rep ("", nBars)
  for (b in 1:nBars) {
    low <- AnzStrats * (nBars-b) / nBars
    up <- AnzStrats * (nBars-b+1.001) / nBars
    relFreq[b] <- sum(data[,treat]>=low & data[,treat]<up) / AnzSubj
    names[b] <- paste (AnzStrats-round(up), "-", AnzStrats-round(low)-1, sep="")
  }
  
  barMatr <- array (0, c(8, nBars))
  barMatr[1,] <- relFreq
  barMatr[2,] <- GetBarProbs (parsGaussOLE, calcGaussOLE, treat, nBars, 7)
  barMatr[3,] <- GetBarProbs (parsCoxCoeff, calcCoxCoeff, treat, nBars, 7)
  barMatr[4,] <- GetBarProbs (parsNestCoeff, calcNestCoeff, treat, nBars, 7)
#  barMatr[5,] <- GetBarProbs (parsNumNestCoeff, calcNumNestCoeff, treat, nBars, 7)
  barMatr[5,] <- GetBarProbs (parsStructRegr, calcStructRegr, treat, nBars, 7)
  barMatr[6,] <- GetBarProbs (parsStructBehav, calcStructBehav, treat, nBars, 7)
  barMatr[7,] <- GetBarProbs (parsTobitRegr, calcTobitRegr, treat, nBars, 7)
  barMatr[8,] <- GetBarProbs (parsTobit2Regr, calcTobit2Regr, treat, nBars, 7)

  barplot (barMatr, beside=TRUE, names.arg=names, xlab="Tokens donated", ylab="Relative Frequencies", main=paste("Treatment", treat), ylim=c(0,.7))
  
  par (xaxs="r", yaxs="r", bty="l", xpd=NA)
  
  if (treat==1) {
    colors <- gray.colors(8)
    inset <- c(0.1, 0.1)

    legend (10, 1.3, c("Empirical", "Ordered GEV", "Rand Taste", "Nested Logit",
                       "Struct Regr", "Rand Behav", "Regression", "Extend. Regr."),
            fill=colors, inset=inset, cex=1.1, ncol=4)
  }

#  legend (111,-0.35, paste("Type", 1:7), lty=1:7, cex=1, horiz=!TRUE, ncol=2)
  
  return (barMatr)
}

BarMatrix2 <- function (filename="") {
  if (filename != "") pdf (filename, width=7, height=12)
  layout (rbind(c(9),c(1,2),c(3,4),c(5,6),c(7,8)))

  for (treat in 1:AnzTreat) 
    m <- PlotBarMatrix2 (treat)
  
  if (filename != "") dev.off ()
}

#######################################################################################


PlotBarMatrix3 <- function (treat) {
  RefData <- read.table (file="garp-dat.csv", header=T, sep=",")
  data <- RefData[,2:9]

  nBars <- 10
  AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
  strats <- (1:AnzStrats - 1) * Step[treat]
  
  relFreq <- rep (0, nBars)
  names <- rep ("", nBars)
  for (b in 1:nBars) {
    low <- AnzStrats * (nBars-b) / nBars
    up <- AnzStrats * (nBars-b+1.001) / nBars
    relFreq[b] <- sum(data[,treat]>=low & data[,treat]<up) / AnzSubj
    names[b] <- paste (AnzStrats-round(up), "-", AnzStrats-round(low)-1, sep="")
  }
  
  barMatr <- array (0, c(3, nBars))
  barMatr[1,] <- relFreq
  barMatr[2,] <- GetBarProbs (parsGaussOLE, calcGaussOLE, treat, nBars, 7)
  barMatr[3,] <- GetBarProbs (parsTobitRegr, calcTobitRegr, treat, nBars, 7)

  barplot (barMatr, beside=TRUE, names.arg=names, xlab="Tokens donated", ylab="Relative Frequencies", main=paste("Treatment", treat), ylim=c(0,.7))
  
  par (xaxs="r", yaxs="r", bty="l", xpd=NA)
  
  if (treat==1 || treat==5) {
    colors <- gray.colors(3)
    inset <- c(0.1, 0.1)

    legend (22, 1.2, c("Empirical", "Ordered GEV", "Regression"),
            fill=colors, inset=inset, cex=2, ncol=3)
  }

#  legend (111,-0.35, paste("Type", 1:7), lty=1:7, cex=1, horiz=!TRUE, ncol=2)
  
  return (barMatr)
}

BarMatrix3 <- function (filename="") {
  
  fname <- paste (filename, "1.pdf", sep="")
  if (filename != "") pdf (fname, width=12, height=9)
  layout (rbind(c(5),c(1,2),c(3,4)))
  for (treat in 1:4) m <- PlotBarMatrix3 (treat)
  if (filename != "") dev.off ()

  fname <- paste (filename, "2.pdf", sep="")
  if (filename != "") pdf (fname, width=12, height=9)
  layout (rbind(c(5),c(1,2),c(3,4)))
  for (treat in 5:8) m <- PlotBarMatrix3 (treat)
  if (filename != "") dev.off ()
}

#######################################################################################


PlotBarMatrix4 <- function (treat) {
  RefData <- read.table (file="garp-dat.csv", header=T, sep=",")
  data <- RefData[,2:9]

  nBars <- 10
  AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
  strats <- (1:AnzStrats - 1) * Step[treat]
  
  relFreq <- rep (0, nBars)
  names <- rep ("", nBars)
  for (b in 1:nBars) {
    low <- AnzStrats * (nBars-b) / nBars
    up <- AnzStrats * (nBars-b+1.001) / nBars
    relFreq[b] <- sum(data[,treat]>=low & data[,treat]<up) / AnzSubj
    names[b] <- paste (AnzStrats-round(up), "-", AnzStrats-round(low)-1, sep="")
  }
  
  barMatr <- array (0, c(4, nBars))
  barMatr[1,] <- relFreq
  barMatr[2,] <- GetBarProbs (parsGaussOLE, calcGaussOLE, treat, nBars, 7)
  barMatr[3,] <- GetBarProbs (parsStructBehav, calcStructBehav, treat, nBars, 7)
  barMatr[4,] <- GetBarProbs (parsTobitRegr, calcTobitRegr, treat, nBars, 7)

  barplot (barMatr, beside=TRUE, names.arg=names, xlab="Tokens donated", ylab="Relative Frequencies", main=paste("Treatment", treat), ylim=c(0,.7))
  
  par (xaxs="r", yaxs="r", bty="l", xpd=NA)
  
  if (treat==1) {
    colors <- gray.colors(4)
    inset <- c(0.1, 0.1)

    legend (10, 1.3, c("Empirical", "Ordered GEV", "Rand Behav", "Regression"),
            fill=colors, inset=inset, cex=1.1, ncol=4)
  }

#  legend (111,-0.35, paste("Type", 1:7), lty=1:7, cex=1, horiz=!TRUE, ncol=2)
  
  return (barMatr)
}

BarMatrix4 <- function (filename="") {
  if (filename != "") pdf (filename, width=7, height=12)
  layout (rbind(c(9),c(1,2),c(3,4),c(5,6),c(7,8)))

  for (treat in 1:AnzTreat) 
    m <- PlotBarMatrix4 (treat)
  
  if (filename != "") dev.off ()
}

#######################################################################################

GetCumProbs <- function (parsFunc, calcFunc, treat, AnzTypes) {
  pars <- parsFunc (AnzTypes, treat)
  shares <- pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]
  AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1

  AggProbs <- array (0, c(AnzStrats))
  for (type in 1:AnzTypes)  {
    StratProbs <- calcFunc (AnzTypes, type, treat)
    AggProbs <- AggProbs + shares[type] * StratProbs
  }

  cumProbs <- rep (AggProbs[1], c(AnzStrats))
  for (i in 2:AnzStrats)
    cumProbs[i] <- cumProbs[i-1] + AggProbs[i]

  return (cumProbs)
}


PlotCumProbs <- function (treat) {
  RefData <- read.table (file="garp-dat.csv", header=T, sep=",")
  data <- RefData[,2:9]

  AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
  strats <- (1:AnzStrats - 1) * Step[treat]

  relFreq <- rep (0, AnzStrats)
  cumFreq <- rep (0, AnzStrats)
  for (s in 1:AnzStrats) {
    relFreq[s] <- sum(data[,treat]==AnzStrats-s) / AnzSubj
    if (s==1) cumFreq[s] <- relFreq[s]
    else cumFreq[s] <- cumFreq[s-1] + relFreq[s]
  }
    
  cumMatr <- array (0, c(4, AnzStrats))
  cumMatr[1,] <- cumFreq
  cumMatr[2,] <- GetCumProbs (parsGaussOLE, calcGaussOLE, treat, 7)
#  cumMatr[3,] <- GetCumProbs (parsCoxCoeff, calcCoxCoeff, treat, 7)
  cumMatr[3,] <- GetCumProbs (parsStructBehav, calcStructBehav, treat, 7)
  cumMatr[4,] <- GetCumProbs (parsTobitRegr, calcTobitRegr, treat, 7)
#  cumMatr[6,] <- GetCumProbs (parsTobit2Regr, calcTobit2Regr, treat, 7)

#  print (cumMatr)
  
  par (xaxs="r", yaxs="r", bty="l", xpd=NA)

  plot (c(min(strats),max(strats)), c(0,1), xlab="Donation to recipient", ylab="Predicted cumulative probability", main=paste("Treatment", treat), type="n")


  for (type in 1:4) lines (strats, cumMatr[type,], lty=type)
  
  if (treat==1) {
    inset <- c(0.1, 0.1)

      legend (70,1.8, c("Empirical", "Ordered GEV", "Rand Behav", "Regression"),
              lty=1:4, cex=1, horiz=!TRUE, ncol=4)
  }

#  legend (111,-0.35, paste("Type", 1:7), lty=1:7, cex=1, horiz=!TRUE, ncol=2)
  
  return (cumMatr)
}

CumPlots <- function (filename="") {
  if (filename != "") pdf (filename, width=7, height=12)
  layout (rbind(c(9),c(1,2),c(3,4),c(5,6),c(7,8)))

  for (treat in 1:AnzTreat) 
    m <- PlotCumProbs (treat)
  
  if (filename != "") dev.off ()
}

#######################################################################################


calcMeanSDev <- function (parsFunc, calcFunc, treat, AnzTypes) {
  pars <- parsFunc (AnzTypes, treat)
  shares <- pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]
  AnzStrats <- (Up[treat] - Low[treat]) / Step[treat]

  AggProbs <- array (0, c(AnzStrats+1))
  for (type in 1:AnzTypes)  {
    StratProbs <- calcFunc (AnzTypes, type, treat)
    AggProbs <- AggProbs + shares[type] * StratProbs
  }

  predMean <- sum (AggProbs * AnzStrats:0)
  predM2 <- sum (AggProbs * (AnzStrats:0)^2)
  predSD <- sqrt (predM2 - predMean^2)

  return (c(predMean, predSD))
}

PlotMeanSDevs <- function (treat) {
  RefData <- read.table (file="garp-dat.csv", header=T, sep=",")
  data <- RefData[,2:9]

  empMean <- mean (data[,treat])
  empSD <- sd (data[,treat])
  
  mdMatr <- array (0, c(7, 2))
  mdMatr[1,] <- calcMeanSDev (parsGaussOLE, calcGaussOLE, treat, 7)
  mdMatr[2,] <- calcMeanSDev (parsCoxCoeff, calcCoxCoeff, treat, 7)
  mdMatr[3,] <- calcMeanSDev (parsNestCoeff, calcNestCoeff, treat, 7)
#  mdMatr[4,] <- calcMeanSDev (parsNumNestCoeff, calcNumNestCoeff, treat, 7)
  mdMatr[4,] <- calcMeanSDev (parsStructRegr, calcStructRegr, treat, 7)
  mdMatr[5,] <- calcMeanSDev (parsStructBehav, calcStructBehav, treat, 7)
  mdMatr[6,] <- calcMeanSDev (parsTobitRegr, calcTobitRegr, treat, 7)
  mdMatr[7,] <- calcMeanSDev (parsTobit2Regr, calcTobit2Regr, treat, 7)

  par (xaxs="r", yaxs="r", bty="o", xpd=NA)
  
  AnzStrats <- (Up[treat] - Low[treat]) / Step[treat]
  plot (c(-7,7), c(-5,5), xlab="Deviation from Mean", ylab="Deviat. from Standard Dev.",
        main=paste("Treatment", treat, "  ( M =", round (AnzStrats-empMean, digits=1), ", SD =", round(empSD, digits=1), ")"),
        type="n")

  points (-c(0, mdMatr[,1]-empMean), c(0, mdMatr[,2]-empSD), pch=2:9)

  if (treat==1) {
    colors <- gray.colors(8)
    inset <- c(0.1, 0.1)

    legend (-1, 15, c("Empirical", "Ordered GEV", "Rand Taste", "Nested Logit",
                     "Struct Regr", "Rand Behav", "Regression", "Extend. Regr."),
            pch=2:9, inset=inset, cex=1.1, ncol=4)
  }

#  legend (111,-0.35, paste("Type", 1:7), lty=1:7, cex=1, horiz=!TRUE, ncol=2)
}

PrintMeanSDevs <- function (filename="") {
  if (filename != "") pdf (filename, width=7, height=12)
  layout (rbind(c(9),c(1,2),c(3,4),c(5,6),c(7,8)))

  for (treat in 1:AnzTreat) PlotMeanSDevs (treat)
  
  if (filename != "") dev.off ()
}

#######################################################################################

IndivClass <- function (parsFunc, calcFunc, AnzTypes) {

  IndivLL <- array (1, c(AnzSubj, AnzTypes))
  for (treat in 1:AnzTreat) {
    for (type in 1:AnzTypes) {
      StratProbs <- calcFunc (AnzTypes, type, treat)
      for (subj in 1:AnzSubj) {
        strat <- data[subj,treat]
        NormStrat <- round((Up[treat] - Low[treat])/Step[treat]) - strat + 1
        IndivLL[subj,type] <- IndivLL[subj,type] * StratProbs[NormStrat]
      }
    }
  }
  
  pars <- parsFunc (AnzTypes)
  shares <- pars[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]
  for (subj in 1:AnzSubj) {
    IndivLL[subj,] <- shares * IndivLL[subj,] / sum (shares * IndivLL[subj,])
  }

  return (IndivLL)
}

PrintIndivClass <- function (parsFunc, calcFunc, AnzTypes) {

  cl <- IndivClass (parsFunc, calcFunc, AnzTypes)
  print (round(cl, digits=3))

  m <- rep (0,AnzSubj)
  for (i in 1:AnzSubj) m[i] <- max(cl[i,])

  cat ("\nMax(EP)<0.99 bei ", sum(m<0.99), "/", AnzSubj, "=", round (sum(m<0.99)/AnzSubj, digits=3), "Subjects\n")
  cat ("\nMax(EP)<0.95 bei ", sum(m<0.95), "/", AnzSubj, "=", round (sum(m<0.95)/AnzSubj, digits=3), "Subjects\n")
  cat ("\nMax(EP)<0.90 bei ", sum(m<0.90), "/", AnzSubj, "=", round (sum(m<0.90)/AnzSubj, digits=3), "Subjects\n")
}

#######################################################################################


printWithinTypes <- function (calcFunc, treat) {
  AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
  
  plot (c(0, AnzStrats), c(0, AnzStrats/3), type="n", xlab="Mean (tokens donated)", ylab="Standard Deviation", main=paste("Treatment", treat))

  typesVec <- 2*1:4-1
  
  for (AnzTypes in typesVec) {
    for (type in 1:AnzTypes){
      StratProbs <- calcFunc (AnzTypes, type, treat)

      mu <- sum (StratProbs * (1:AnzStrats - 1))
      exp2 <- sum (StratProbs * (1:AnzStrats - 1)^2)
      sd <- sqrt(exp2 - mu^2)
      points (mu, sd, pch=c(AnzTypes+1))
    }
  }

  if (treat==1)
    legend (32,-4.5, paste("Number of Types =", typesVec), pch=typesVec-1, cex=1.3, horiz=!TRUE, ncol=1)
}

printTypePics1 <- function(calcFunc, filename="") {
  if (filename != "") pdf (filename, width=7, height=9.5)
  layout (rbind(c(1,2),c(3,4),c(5,6),c(7,8)))
  par (xaxs="r", yaxs="r", bty="o", xpd=NA)

  for (treat in 1:AnzTreat) 
    printWithinTypes (calcFunc, treat)
  
  if (filename != "") dev.off ()
}

#######################################################################################

printBetweenTypes <- function (treat) {
  AnzStrats <- (Up[treat] - Low[treat]) / Step[treat] + 1
  
  plot (c(0, AnzStrats), c(0, AnzStrats/2), type="n", xlab="Mean (tokens donated)", ylab="Standard Deviation", main=paste("Treatment", treat))

  AnzTypes <- 7

  ct <- 1
  for (calcFunc in c(calcGaussOLE, calcCoxCoeff, calcStructBehav, calcTobitRegr)) {
    for (type in 1:AnzTypes){
      StratProbs <- calcFunc (AnzTypes, type, treat)
    
      mu <- sum (StratProbs * (1:AnzStrats - 1))
      exp2 <- sum (StratProbs * (1:AnzStrats - 1)^2)
      sd <- sqrt(exp2 - mu^2)
      points (mu, sd, pch=c(ct+1))
    }
    ct <- ct+1
  }

  if (treat==1)
    legend (32,-10, c("Rand Utility", "Rand Taste", "Rand Behav", "Regression"), pch=2:5, cex=1.3, horiz=!TRUE, ncol=2)
}

printTypePics2 <- function(filename="") {
  if (filename != "") pdf (filename, width=7, height=9.5)
  layout (rbind(c(1,2),c(3,4),c(5,6),c(7,8)))
  par (xaxs="r", yaxs="r", bty="o", xpd=NA)

  for (treat in 1:AnzTreat) 
    printBetweenTypes (treat)
  
  if (filename != "") dev.off ()
}

#######################################################################################


if (1) {
  FlInSample <- TRUE
  ExPostClass (parsGaussOLE, calcGaussOLE, 7, "garp-ep-1.pdf")

  FlInSample <- TRUE ; BarMatrix ("garp-bar-in.pdf")
  FlInSample <- FALSE ; BarMatrix ("garp-bar-out.pdf")

  FlInSample <- TRUE ; BarMatrix2 ("garp-bar2-in.pdf")
  FlInSample <- FALSE ; BarMatrix2 ("garp-bar2-out.pdf")
  
  FlInSample <- FALSE ; BarMatrix3 ("garp-bar3-")
  
  FlInSample <- TRUE ; BarMatrix4 ("garp-bar4-in.pdf")
  FlInSample <- FALSE ; BarMatrix4 ("garp-bar4-out.pdf")

  FlInSample <- TRUE ; CumPlots ("garp-cum-in.pdf")
  FlInSample <- FALSE ; CumPlots ("garp-cum-out.pdf")

  FlInSample <- TRUE ; PrintMeanSDevs ("garp-msd-in.pdf")
  FlInSample <- FALSE ; PrintMeanSDevs ("garp-msd-out.pdf")
  
  FlInSample <- TRUE
  printTypePics1 (calcGaussOLE, "type-pic-OGEV.pdf")
  printTypePics2 ("type-pic-2.pdf")
}

if (0) PrintIndivClass (parsGaussOLE, calcGaussOLE, 7)

#PrintMeanSDevs ("garp-msd-in.pdf")
#BarMatrix ("garp-bar-in.pdf")
#BarMatrix2 ("garp-bar2-in.pdf")

#FlInSample <- FALSE ; BarMatrix4 ("garp-bar4-out.pdf")
