
RefData <- read.table (file="Fisman-two-person.csv", header=T, sep=",")
AnzTreat <- max(RefData$Round)    # 50
AnzSubj <- max(RefData$ID)        # 76
AnzDat <- rep (AnzSubj, AnzTreat)


RefData$Low <- 0
RefData$Up <- RefData$Max.Hold*10
RefData$Step <- 1
RefData$Rate <- RefData$Max.Pass / RefData$Max.Hold

RefData$NormHold <- round(10*RefData$Hold * RefData$Max.Hold / (RefData$Hold + RefData$Pass/RefData$Rate))
RefData$NormPass <- pmax (0, RefData$Up - RefData$NormHold)

#round(10*RefData$Pass * RefData$Max.Hold / (RefData$Hold + RefData$Pass/RefData$Rate))

TransData <- t(RefData)


####################################################################


ratioV <- c(4/1, 3/1, 5/2, 2/1, 3/2, 1/1, 1/2, 1/3)
ratioLab <- c("4:1", "3:1", "5:2", "2:1", "3:2", "1:1", "1:2", "1:3")

library (reshape2)
library (ggplot2)
theme_set (theme_gray())

sortRatioV <- c()
meanV <- c()
sdV <- c()
mean2V <- c()
for (i in length(ratioV):1) {
    r <- ratioV[i]
    d <- RefData[RefData$Rate>r/1.1 & RefData$Rate<r*1.1,]

    s <- (d$Pass / d$Rate) / (d$Hold + d$Pass / d$Rate)

    cat (ratioLab[i], ": Num =", length(s), "\tMean =", mean(s), "\tMean|s>0 =", mean(s[s>0]), "\tSD =", sd(s), "\n")

    sortRatioV <- c(sortRatioV, ratioV[i])
    meanV <- c(meanV, mean(s))
    mean2V <- c(mean2V, mean(s[s>0]))
    sdV <- c(sdV, sd(s))
    mainStr <- paste("FKM07: Transfer ", ratioLab[i], sep="")

    filename <- paste("hist-fis2-", i, ".pdf", sep="")
#    pdf (filename, width=9, height=6)
#    par(mar=c(5,4.5,4,2) + 0.1)
    
#    hist(s,
#         xlim=c(0,1), #ylim=c(0,120),
#         breaks=0:100/100,
#         main="",
#         main=list(paste("Transfer ", ratioLab[i], sep=""), cex=2.2),
#         xlab=list("Budget share transfered", cex=1.8),
#         ylab=list("Percentage of choices", cex=1.8),
#         freq=FALSE,
#         cex.axis=1.8)
    
    dat <- data.frame (s=s, title=as.factor(rep(mainStr, length(s))))
    ggplot(dat, aes (s)) + labs(x=xlabStr, y="Relative frequency") + geom_histogram(aes(y=..count../sum(..count..)), binwidth=0.01) + facet_grid(. ~ title) + ylim(0,1/3) +
        theme(strip.text.x = element_text(size = 22),
              # axis.title.y=element_text(size=22), axis.text.y=element_text(size=18),
              axis.title.y=element_blank(), axis.text.y=element_blank(), axis.ticks.y=element_blank(),
              axis.title.x=element_text(size=22), axis.text.x=element_text(size=18)
              )
    ggsave (filename, device="pdf")
    
#    par(oma=c(2,2,2,2))
#    dev.off()
    
}

cat ("ratioFis <- c(", paste(sortRatioV, collapse=","), ")\n")
cat ("meanFis <- c(", paste(meanV, collapse=","), ")\n")

##########################################################################################


ratioFis <- c( 0.333333333333333,0.5,1,1.5,2,2.5,3,4 )
meanFis <- c( 0.158056168003668,0.187916924677542,0.187023567706353,0.197293554846818,0.217551973026782,0.2069825626759,0.227502927117548,0.205364451811775 )

ratioGarp <- c( 0.333333333333333,0.5,1,2,3 )
meanGarp <- c( 0.200568181818182,0.209043560606061,0.236401515151515,0.312859848484848,0.3203125 )

ratioHarr <- c( 1,1.5,2,2.5,3,4 )
meanHarr <- c( 0.290594541910331,0.332163742690058,0.333745126705653,0.307456140350877,0.31359649122807,0.306140350877193 )

td <- test_data <- data.frame (ratio=c(ratioFis, ratioGarp, ratioHarr), means=c(meanFis, meanGarp, meanHarr), t=c(rep("FKM07", length(meanFis)), rep("AM02", length(meanGarp)), rep("HJ06", length(meanHarr))))

test_data_long <- melt(test_data, id="t")
    
filename <-"compar-all.pdf"
ggplot() + xlim(0,4) + ylim(0.05,0.35) +
    geom_line(aes(x = td$ratio[td$t=="FKM07"], y = td$means[td$t=="FKM07"], color = "FKM07"))  +
    geom_line(aes(x = td$ratio[td$t=="AM02"], y = td$means[td$t=="AM02"], color = "AM02")) +
    geom_line(aes(x = td$ratio[td$t=="HJ06"], y = td$means[td$t=="HJ06"], color = "HJ06")) +
    labs(title="", x="Ratio Value Recipient / Value Dictator", y="Budget share transfered", color="") +
    theme(plot.title = element_text(size=25), axis.title.y=element_text(size=22), axis.text.y=element_text(size=18),
          legend.text=element_text(size=20), legend.title=element_text(size=25), legend.spacing=unit(10,"mm"),
          axis.title.x=element_text(size=22), axis.text.x=element_text(size=18))
ggsave (filename, device="pdf", width=11, height=7)
    
#pdf (filename, width=9, height=6)
#par(mar=c(5,4.5,4,2) + 0.1)
#plot(c(0,4), c(0, 0.4), type="n",
#     main="",
#     xlab=list("Ratio Value Recipient / Value Me", cex=1.8),
#     ylab=list("Budget share transfered", cex=1.8),
#     cex.axis=1.8)
#lines (ratioGarp, meanGarp, lty=1)
#lines (ratioHarr, meanHarr, lty=2)
#lines (ratioFis, meanFis, lty=4)
#legend ("topleft", legend=c("AM", "HJ", "FKM"), lty=c(1,2,4))

#par(oma=c(2,2,2,2))
#dev.off()


cat ("\n\n\\hline \\hline\n")
cat (" &", paste(round(ratioFis, digits=2), collapse=" & "), "\\\\ \\hline\n")

cat ("AM")
for (i in 1:length(ratioFis)) {
    if (sum(ratioGarp==ratioFis[i])==0) cat (" & ")
    else {
        r <- meanGarp[ratioGarp==ratioFis[i]]
        cat (" & ", round (r, digits=3))
    }
}
cat ("\\\\ \n")

cat ("HJ")
for (i in 1:length(ratioFis)) {
    if (sum(ratioHarr==ratioFis[i])==0) cat (" & ")
    else {
        r <- meanHarr[ratioHarr==ratioFis[i]]
        cat (" & ", round (r, digits=3))
    }
}
cat ("\\\\\n")

cat ("FKM")
for (i in 1:length(ratioFis)) {
    if (sum(ratioFis==ratioFis[i])==0) cat (" & ")
    else {
        r <- meanFis[ratioFis==ratioFis[i]]
        cat (" & ", round (r, digits=3))
    }
}
cat ("\\\\\n \\hline\\hline\n")

