
prepInvMatr <- function (matr, estim) {
  degen <- c()
  FL <- 1
  while (FL) {
    degenOld <- degen
    max <- 0
    for (k in 1:nrow(matr)) 
      if (!(k %in% degen) && sum(abs(matr[k,]))>max) max <- sum(abs(matr[k,]))
    for (k in 1:nrow(matr)) 
      if (sum(abs(matr[k,]))<1e-5) { degen <- c(degen,k) }
    
    matr[,degen] <- 0
    matr[degen,] <- 0
    diag(matr)[degen] <- 1
    serr <- try (diag(solve (matr)), TRUE)
    if (!inherits(serr, "try-error")) break

    for (k in 1:nrow(matr)) 
      if (sum(abs(matr[k,]))==max) { degen <- c(degen,k) }
    
    matr[,degen] <- 0
    matr[degen,] <- 0
    diag(matr)[degen] <- 1

#    cat ("Degen: ", degen, "\n")
    FL <- ifelse (length(degen) > length(degenOld), 1, 0)
  }
  diag(matr)[degen] <- 1

  return (list(matr=matr, degen=degen))
}

DiagInvMatr <- function (matr, estim) {
  prepM <- prepInvMatr (matr, estim)
  matr <- prepM$matr
  degen <- prepM$degen
  
  serr <- try (diag(solve (matr)), TRUE)
  if (inherits(serr, "try-error")) serr <- rep(NaN, length(estim))
  else serr <- sqrt(abs(serr))
  
  serr[degen] <- NaN
  return (serr)
}

calcCLC <- function (baseLL) { # assumes lastIndivLL & lastPostClass to be set by previous call of GetLL
     # baseLL = log-likelihood  =>  result is CLC
     # baseLL = BIC  =>  result is ILC-BIC

  en <- 0
  for (subj in 1:AnzSubj) {
    pc <- pmax (1e-10, lastPostClass[subj,])
    lastCLC[subj] <<- lastIndivLL[subj] + sum (pc * log (pc))
    en <- en - sum (pc * log (pc))
  }
  
  return (baseLL + en)
}

calcIndivGradM <- function (res) {

    indivGradM <- array (0, c(AnzSubj, length(res$estimate)))
    
    for (i in 1:length(res$estimate)) {
        pars2 <- res$estimate
        delta <- pmin(1e-2, pmax (1e-3, res$estimate[i] / 1e3))

        pars2[i] <- pars2[i] - delta/2
        ll <- GetLL (pars2, RelTr=RelTreats)
        llIndiv1 <- lastIndivLL

        pars2[i] <- pars2[i] + delta
        ll <- GetLL (pars2, RelTr=RelTreats)
        llIndiv2 <- lastIndivLL

        indivGradM[,i] <- (llIndiv2 - llIndiv1) / delta
    }

    res$indivGradM <- indivGradM

    return (res)
}

calcHuberSandwich <- function (res) {

    V <- array (0, c(length(res$estimate), length(res$estimate)))

    for (s in 1:AnzSubj) {
        Vs <- res$indivGradM[s,] %*% t(res$indivGradM[s,])
        V <- V + Vs
    }
    
    prepM <- prepInvMatr (res$hessian, res$estimate)
    matr <- prepM$matr
    degen <- prepM$degen
  
    H <- try (solve (matr), TRUE)
    if (inherits(H, "try-error")) return (rep(NaN, length(estim)))

    HVH <- H %*% V %*% H

    serr <- sqrt(abs(diag(HVH)))
    serr[degen] <- NaN

    return (serr)
}

PrintStuff <- function (res, extra=c()) {

  varLabV <- c("Share", "Lambda", "Rho", "Sig-Rho", "Alpha", "Beta")
  varFLV <- c(TRUE, TRUE, ifelse(M>0, TRUE, FALSE), ifelse(MixMode %in% 4:5 && M>0, TRUE, FALSE), TRUE, TRUE)
  if (AnzTypePars==7){
      varLabV <- c(varLabV, "Sig-Alpha", "Sig-Beta")
      varFLV <- c(varFLV, TRUE, TRUE)
  } else   if (AnzTypePars==8){
      varLabV <- c(varLabV, "Sig-Alpha", "Sig-Beta", "Sig-Lambda")
      varFLV <- c(varFLV, TRUE, TRUE, TRUE)
  }
    
  ind <- c()
  for (t in 1:AnzTypes)
    ind <- c (ind, paste (varLabV, t, sep="-"))

  if (!exists ("M")) M <- 0

  FL <- rep(varFLV, AnzTypes)

  res$estimate <- NormalizeTypeShares (res$estimate)
  AnzPar <- sum(FL) - 1

  bic <- res$minimum + AnzPar/2 * log(AnzObs)

  ll <- GetLL (res$estimate, RelTr=RelTreats)  # called to set lastPostClass
  clc <- calcCLC (bic)

  if (1) {
      if (is.null(res$indivGradM)) res <- calcIndivGradM (res)
      res$se <- calcHuberSandwich (res)
      save (res, file=ResFile)
      
  } else if (is.null(res$se)) {
      res$se <- DiagInvMatr (res$hessian, res$estimate)
      save (res, file=ResFile)
  }
  
  cat ("############################################################################\n",
       paste (paste ("Fold =", FoldNum), paste("M =", M), paste("AnzTypes =", AnzTypes), paste("LL =", round (-res$minimum, digits=2)), paste("BIC =", round(bic, digits=2)), paste("CLC =", round(clc, digits=2)), "\n", paste ("AnzSubj = ", AnzSubj), paste ("AnzPar = ", AnzPar), sep="      "), "\n\n")

#    print (res)
    
  StandErr1 <- res$se

  cat (paste (paste(ind[FL], ": "), round(res$estimate[FL], digits=4), paste("(", round(StandErr1[FL], digits=4), ")", sep=""), sep="\t", collapse="\n"), "\n\n")

  if (length(RelTreats)==AnzTreat) {
    LL0 <- GetLL (res$estimate, RelTr=1:AnzTreat)
    LL1 <- GetLL (res$estimate, RelTr=InTreats1)
    LL2 <- GetLL (res$estimate, RelTr=OutTreats1)
    clc2 <- calcCLC (LL2)
  } else {
    LL0 <- GetLL (res$estimate, RelTr=1:AnzTreat)
    LL1 <- GetLL (res$estimate, RelTr=RelTreats)

    complTr <- setdiff (1:AnzTreat, RelTreats)
    LL2 <- GetLL (res$estimate, RelTr=complTr)
    clc2 <- calcCLC (LL2)
  }
  cat ("LLs overall:", round(-c(LL0, LL1, LL2), digits=2), "\n\n")

  if (length(RelTreats)==AnzTreat) savedBics[AnzTypes] <<- bic
  else savedBics[AnzTypes] <<- LL2

  if (length(RelTreats)==AnzTreat) savedCLC[AnzTypes] <<- clc
  else savedCLC[AnzTypes] <<- clc2
  
  cat ("Termination code =", res$code, "\tGradient =", paste (round(res$gradient[FL], digits=3), collapse=" "), "\n\n")

  cat ("op <- c(", paste(round(res$estimate, digits=5), collapse=","), ")\n\n")

  if (1) {
    x <- round (res$estimate, digits=3)
    se <- paste("(", round(StandErr1, digits=3), ")", sep="")
    x[!FL] <- ""
    se[!FL] <- ""
    types <- 1:AnzTypes

#    LL <-  paste ("\\drA{", round(ll, digits=2), "}{", round(bic, digits=2), "}{", round(clc, digits=2), "}", sep="")
    LL <-  round (ll, digits=2)
    
    ATout <- c("\\textit{One type}", "\\textit{Two types}", "\\textit{Three types}", "\\textit{Four types}", "\\textit{Five types}", "\\textit{Six types}", "\\textit{Seven types}", "\\textit{Eight types}", "\\textit{Nine types}")[AnzTypes]
    ATout <- c(ATout, rep(" ", AnzTypes-1))

#    if (M==0) fl <- c(TRUE, TRUE, FALSE, FALSE, rep(TRUE, AnzTypePars-4+1))
#    if (M==1) fl <- c(TRUE, TRUE, TRUE, FALSE, rep(TRUE, AnzTypePars-4+1))
#    if (M==2) fl <- c(TRUE, TRUE, TRUE, TRUE, rep(TRUE, AnzTypePars-4+1))

    unOut <- paste ("\\un{", se, "}{", x, "}")
    unOut[!FL] <- ""

    texLine <- ""
    for (t in 1:AnzTypes) {
        
        if (AnzTypes>1) { # Full, with number of types, type number, shares
            tpInd <- c(1:(AnzTypePars+1) + (t-1)*(AnzTypePars+1))
            texLine <- paste (texLine, ATout[t], " & ", t, paste (" & ", unOut[tpInd], collapse=""), ifelse (t<AnzTypes, "\\\\\n", ""))          
            
        } else {
            tpInd <- c(2:(AnzTypePars+1) + (t-1)*(AnzTypePars+1))
            texLine <- paste (texLine, " & ", paste (" & ", unOut[tpInd], collapse=""), ifelse (t<AnzTypes, "\\\\\n", ""))
        }
    }
    
    texLine <- paste (texLine, paste (rep ("&", 9-AnzTypePars), collapse=" "), LL, "\\\\\n")

    #cat (texLine)

    zz <- file(TexFile, "a")
    cat(texLine, file = zz)
    close(zz)

    shares <- x[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]
    alpha <- x[5 + (1:AnzTypes - 1) * (AnzTypePars+1)]
    beta <- x[6 + (1:AnzTypes - 1) * (AnzTypePars+1)]

    if (AnzTypePars>5) {
        sigAlpha <- x[7 + (1:AnzTypes - 1) * (AnzTypePars+1)]
        sigBeta <- x[8 + (1:AnzTypes - 1) * (AnzTypePars+1)]
    }

    if (AnzTypePars<=5) {
      texLine2 <- paste (" && \\parc{", shares, "\\newline $", alpha, "$\\newline $", beta, "$} ", sep="", collapse="")
    } else {
      texLine2 <- ""
      for (t in 1:AnzTypes)
        if (t %in% MixTypesV || t == AnzTypes)
          texLine2 <- paste (texLine2, paste (" && \\parc{", shares[t], "\\newline $", alpha[t], ",", sigAlpha[t], "$\\newline $", beta[t], ",", sigBeta[t], "$} ", sep=""))
        else
          texLine2 <- paste (texLine2, paste (" && \\parc{", shares[t], "\\newline $", alpha[t], "$\\newline $", beta[t], "$} ", sep=""))
    }

    zz <- file (paste (TexFile, AnzTypes, sep=""), "w")
    cat(texLine2, file = zz)
    close(zz)

  }

  cat ("\n##########################################################################\n")
}

if (!exists ("baseParMin")) {
    baseParMin <- c(0, 0, 0.01, 0.3, -1, -100)
    baseParMax <- c(1, 1000, 10, 1000, 1, 100)
}


##################################################################################

calcStartPars <- function (old) {
  new <- rep (0, AnzTypePars+1)

  for (i in 1:(AnzTypePars+1))
    new[i] <- mean (old[i + (2:AnzTypes-2) * (AnzTypePars+1)])
  
  new[1] <- new[1]/8

  sp <- c(new, old)
  shares <- sp[1 + (1:AnzTypes - 1) * (AnzTypePars+1)]
  shares <- shares / sum (shares)

  sp[1 + (1:AnzTypes - 1) * (AnzTypePars+1)] <- shares

  return (sp)
}

