
argsV <- as.double (commandArgs(TRUE))
if (length(argsV)==0) {
    
    cat ("No args\n")
    TablesDoBeDone <- 3   #  1:9
    
} else {

    cat ("args:", argsV, "\n")
    TablesDoBeDone <- argsV
}

cat ("\n")



# 1 - Are utility scales adaptative?
# 2 - How do subjects differ?
# 3 - How do subjects choose transfers?
# 4 - Are stylized facts reproduced?
# 5 - Are parameters robustly identified? Moderate 0
# 6 - Are parameters robustly identified? Moderate 1
# 7 - Are parameters robustly identified? Moderate 2
# 8 - Parameter estimates, Distribution of choices
# 9 - incrMods

###################################################

AnzBootRuns <- 1e3
BootLRTest <- FALSE
library (boot)
library (entropy)

sumDiff <- function (d, w) sum (d*w)

myPValTwo <- function (x1, x0) mean (abs(x1-mean(x1, na.rm=TRUE)) > abs(x0), na.rm=TRUE) + 0.5 * mean (abs(x1-mean(x1, na.rm=TRUE)) == abs(x0), na.rm=TRUE)

NestLR <- function (LLs1, LLs2, ap1, ap2) {
  t0 <- 0.5
  t1 <- 0.5
  t2 <- 0.5

  l <- max (length(LLs1), length(LLs2))
  lFac <- sqrt(log(log(l))) / sqrt(sqrt(l))
  epsV <- rep (1, l+1)
  epsV <- epsV + (0:l %% 2) * lFac

  LLs1 <- LLs1 * epsV[1:l]
  LLs2 <- LLs2 * epsV[1:l + 1]
  
  BIC1 <- sum (LLs1) # + ap1/2 * log(length(LLs1))
  BIC2 <- sum (LLs2) # + ap2/2 * log(length(LLs2))
  
  LLdiff <- BIC1 - BIC2

  lastLLs1 <<- LLs1
  lastLLs2 <<- LLs2
  lastAP1 <<- ap1
  lastAP2 <<- ap2
  
  if (abs (LLdiff) > 0.02) {
      
      if (BootLRTest) {
          set.seed(0)
          b <- boot (LLs1 - LLs2, sumDiff, R=AnzBootRuns, stype="w")
          p <- myPValTwo (b$t, b$t0)
      } else {
          LLdiff <- sum (LLs1 - LLs2)
          p <- 1 - pchisq (2*abs(LLdiff), max(1,abs(ap1-ap2)))
          if (p==0) p <- 1e-16
      }
  } else p <- 0.5
      
  
#  s <- ifelse (LLdiff>0, 1, -1)
  s <- ifelse (abs(sum(LLs1)) > abs(sum(LLs2)), 1, -1)
  
  return (s * p)
}

NonNestLR <- function (LLs1, LLs2, ap1, ap2) {
  p <- 0.5

  origLLs1 <<- LLs1
  origLLs2 <<- LLs2
  
  l <- max (length(LLs1), length(LLs2))
  lFac <- sqrt(log(log(l))) / sqrt(sqrt(l))
  epsV <- rep (1, l+1)
  epsV <- epsV + (0:l %% 2) * lFac

  lastEpsV <<- epsV

  lastLLs1 <<- LLs1 <- LLs1 * epsV[1:l]        #  + ap1/2 * log(length(LLs1)) / length(LLs1)
  lastLLs2 <<- LLs2 <- LLs2 * epsV[1:l+1]      #  + ap2/2 * log(length(LLs2)) / length(LLs2)
  lastAP1 <<- ap1
  lastAP2 <<- ap2
  
  LLdiff <- sum(LLs1 - LLs2)
  
  if (abs (LLdiff) > 0.02 && length(LLs1)!=1 && origLLs2!=-1) {

    if (BootLRTest) {
#      set.seed(0)
      b <- boot (LLs1 - LLs2, sumDiff, R=AnzBootRuns, stype="w")
      p <- myPValTwo (b$t, b$t0)
    } else {
        p <- t.test(LLs1-LLs2)$p.value
#        p <- wilcox.test(LLs1-LLs2)$p.value
    }
    if (p==0) p <- 1e-16
  }

  s <- ifelse (abs(sum(LLs1)) > abs(sum(LLs2)), 1, -1)
  
  return (s * p)
}


testLLs <- function (LLs1, LLs2, ap1, ap2, isNested=FALSE, wideSigns=FALSE, SumUpLL=TRUE, SqrtLL=FALSE, AllowLLs2Zero=FALSE) {

    if (length(LLs1)!=length(LLs2) && (sum(LLs2)!=0 || !AllowLLs2Zero)) {
        return (list (meas1=0, meas2=0, pVal=0.5, relSign=""))
    }

    LLs1[is.na(LLs1)] <- -1e5
    LLs2[is.na(LLs2)] <- -1e5
    if (length(LLs1)==length(LLs2)) zeroInd <- (LLs1==0) & (LLs2==0)
    else zeroInd <- FALSE
    
    lastLLs1 <<- LLs1 <- pmax(-1e5, LLs1[!zeroInd])
    lastLLs2 <<- LLs2 <- pmax(-1e5, LLs2[!zeroInd])
   
  if (SumUpLL) {
      BIC1 <- ifelse (sum(LLs1)!=0, sum (LLs1) + ap1/2 * log(length(LLs1)), 0)
      BIC2 <- ifelse (sum(LLs2)!=0, sum (LLs2) + ap2/2 * log(length(LLs2)), 0)
  } else {
      BIC1 <- 100*mean (LLs1)
      BIC2 <- 100*mean (LLs2)
  }

  if (SqrtLL) { BIC1 <- sqrt(BIC1) ; BIC2 <- sqrt(BIC2) }

  if (sum (LLs1)==0 || (sum (LLs2)==0 && !AllowLLs2Zero) || sum (abs(LLs1-LLs2))==0) {
      return (list (meas1=0, meas2=0, pVal=0.5, relSign=""))
  }

  if (0 && isNested) pV <- NestLR (LLs1, LLs2, ap1, ap2)
  else pV <- NonNestLR (LLs1, LLs2, ap1, ap2)
  
  pVal <- pV[1]

  if (wideSigns) {
#    if (abs(pVal) <= 0.001) RelSign <- ifelse (pVal<0, "$<<<$", "$>>>$") else
    if (abs(pVal) <= 0.005) RelSign <- ifelse (pVal<0, "$<<$", "$>>$")
    else if (abs(pVal) <= 0.05) RelSign <- ifelse (pVal<0, "$<$", "$>$")
    else RelSign <- "$\\approx$"
  } else {
#    if (abs(pVal) <= 0.001) RelSign <- ifelse (pVal<0, "$\\lll$", "$\\ggg$") else
      if (abs(pVal) <= 0.005) RelSign <- ifelse (pVal<0, "$\\ll$", "$\\gg$")
    else if (abs(pVal) <= 0.05) RelSign <- ifelse (pVal<0, "$<$", "$>$")
    else RelSign <- "$\\approx$"
  }
    
  return (list (meas1=BIC1, meas2=BIC2, pVal=pVal, relSign=RelSign))
}


##########################################################################################################


getAdeqFileName <- function (exp, model, MixMode, Moderate) {
    # exp in c("bard", "capp", "fis", "garp", "harr", "list")
    # model in c("logit", "ole", "palm2", "palm3", "prom")
    # MixMode in c(1,3,4,5),   Moderate in 0:2

    if (MixMode >= 0) {
        fn1 <- paste ("../", exp, "-mix-", model, "/Adeq/", sep="")

        if (Moderate==0) num <- c("", 2, 3, 5, 7)[as.double(MixMode)]
        else if (Moderate==1) num <- c(9, 0, 4, 6, 8)[as.double(MixMode)]
        else if (Moderate==2) num <- c(10, 0, 11, 12, 13)[as.double(MixMode)]
        
    
        if (is.na (num)) {
            lastModerate <<- Moderate
            lastMixMode <<- MixMode
            
            cat ("MixMode", MixMode, "\t Moderate", Moderate, "\n")
            XXX
        }

        exp2 <- exp
        if (exp2 == "garp") exp2 <- "opt"
    
        fn2 <- paste (exp2, "-mix", num, "-", model, "-t1.rda", sep="")

        fn <- paste (fn1, fn2, sep="")
        
    } else {
        fn1 <- paste ("../", exp, "-", model, "/Adeq/", sep="")

        exp2 <- exp
        if (exp2 == "garp") exp2 <- "opt"
        fn2 <- paste (exp2, "-", model, "-t1.rda", sep="")

        fn <- paste (fn1, fn2, sep="")
    }

#    cat ("   File (", exp, model, MixMode, Moderate, ") =>", fn, "\n")
    
    return (fn)
}

getAdeqItem <- function (fname, measure, exp="", adeqLoc=NULL) { # does not deal with pooled

    if (!file.exists(fname)) return (-1)
    if (is.null(adeqLoc)) load (fname) else adeq <- adeqLoc

    lastAdeq <<- adeq

    if (!is.null (adeq[[measure]])) {
#        cat ("  ", fname, " - ", measure, "  =>  ", adeq[[measure]], "\n")
        return (abs(adeq[[measure]]))
    }

    relPWa <- abs(getAdeqItem (fname, "PWa", adeqLoc=adeq)) - abs(getAdeqItem (fname, "LLmax", adeqLoc=adeq))

    relBIC <- abs(getAdeqItem (fname, "BIC", adeqLoc=adeq)) - abs(getAdeqItem (fname, "LLmax", adeqLoc=adeq))
    absBIC <- 0  # abs(getAdeqItem (fname, "BIC"))

    if (measure == "CW") return (abs(getAdeqItem (fname, "PW", adeqLoc=adeq)) - relPWa)
    if (measure == "CE") return (abs(getAdeqItem (fname, "EW", adeqLoc=adeq)) - relBIC)

    measCharV <- strsplit (measure, split="")[[1]]
    origMeasure <- paste (c("P", measCharV[2:length(measCharV)]), collapse="")
    
    if (measCharV[2] == "F") return (abs(getAdeqItem (fname, origMeasure, adeqLoc=adeq)) - absBIC)

    if (exp=="") XXX
    
    exp2V <- c("bard", "capp", "fis", "garp", "harr", "list")
    if (length(measCharV)==4) {
        if (measCharV[4] == "a") exp2V <- c("fis", "garp", "harr")
        if (measCharV[4] == "b") exp2V <- c("bard", "capp", "list")
        if (measCharV[4] == "c") exp2V <- c("garp", "harr")
        if (measCharV[4] == "d") exp2V <- c("bard", "capp", "fis", "list")
        if (measCharV[4] == "e") exp2V <- c("capp", "fis", "garp", "harr")
        if (measCharV[4] == "f") exp2V <- c("bard", "list")
        if (measCharV[4] == "g") exp2V <- c("capp", "garp", "harr")
    }
    expr2V <- setdiff (exp2V, exp)
    
    predFac <- length(expr2V)
    if (measCharV[2] == "B") return (abs(getAdeqItem (fname, origMeasure, adeqLoc=adeq)) - predFac*relBIC)

    
    cat ("Did not find measure", measure, "\t(", fname, ")\n")
    
    return (-1)
}

loadAdeqMeasure <- function (exp, model, MixMode, Moderate, measure, scale="", MakeLLNormalization=TRUE) {
    # measure in c("LL", "BIC", "PW", "EW", "PB1", "PF1", "PB2", "PF2", "PB3", "PF3", "PB1e", "PB1f", "PB2e", "PB2f", "PB3e", "PB3f", "CB1", "CF1", "CB2", "CF2", "CB3", "CF3")
    # do appropriate transformation for CB1, CF1, etc

    if (scale=="") scale <- "LL"
    if (scale=="R2" || scale=="L0") MakeLLNormalization <- FALSE

    if (exp == "pool") exp2V <- c("bard", "capp", "fis", "garp", "harr", "list")           # All
    if (exp == "pool2") exp2V <- c("bard", "capp", "garp", "harr", "list")                 # Numerical
    if (exp == "pool3") exp2V <- c("capp", "fis", "garp", "harr")                          # All DG
    if (exp == "pool4") exp2V <- c("fis", "garp", "harr")                                  # Rich DG
    if (exp == "pool5") exp2V <- c("capp", "garp", "harr")                                 # Numerical DG
    if (exp == "pool6") exp2V <- c("bard", "list")                                         # (Numerical) TG
    if (exp == "pool7") exp2V <- c("garp", "harr")                                         # Standard DG
    if (exp == "pool8") exp2V <- c("bard", "capp", "list")                                 # Entitlement DG
    
    measCharV <- strsplit (measure, split="")[[1]]

    if (exp %in% c("pool", "pool2", "pool3", "pool4", "pool5", "pool6", "pool7", "pool8")) {

        m <- 0
        pBic <- 0
        llMax <- 0
        llMin <- 0
        for (exp2 in exp2V) {
            fname <- getAdeqFileName (exp2, model, MixMode, Moderate)
            if (!file.exists(fname)) next

            m2 <- loadAdeqMeasure (exp=exp2, model, MixMode, Moderate, measure, scale="", MakeLLNormalization=MakeLLNormalization)
            x1 <- m2$meas
            x2 <- m2$llMax
            x3 <- m2$llMin

            if (measCharV[2] %in% c("B","F")) m <- m + x1 else m <- m + x1 - MakeLLNormalization * x2
            llMax <- llMax + x2 - MakeLLNormalization * x2
            llMin <- llMin + x3 - MakeLLNormalization * x2

            if (measCharV[1] == "C" && scale %in% c("R2", "L0")) {
                llMin <- llMin - llMax
                llMax <- 0
            }
            
#            cat (exp, "- ", measure, ":", fname, ":", x1, "/", x2, "=>", m, "\n")
        }
        
        if (measCharV[1]=="C" && measCharV[2]=="F") m <- m - pBic
        if (0 && measCharV[1]=="P" && measCharV[2]=="B" && scale=="L0") {
            m <- m / length(exp2V) ; llMax <- llMax / length(exp2V) ; llMin <- llMin / length(exp2V)
        }


    } else {
        fname <- getAdeqFileName (exp, model, MixMode, Moderate)
        if (!file.exists(fname)) return (-1)
        load (fname)

        m <- getAdeqItem (fname, measure, exp, adeqLoc=adeq)
        
        llMax <- getAdeqItem (fname, "LLmax", adeqLoc=adeq)
        llMin <- getAdeqItem (fname, "LLmin", adeqLoc=adeq)
        
        if (measCharV[2] %in% c("F", "B")) {
            
            exp2V <- c("bard", "capp", "fis", "garp", "harr", "list")
            if (length(measCharV)==4) {
                if (measCharV[4] == "a") exp2V <- c("fis", "garp", "harr")
                if (measCharV[4] == "b") exp2V <- c("bard", "capp", "list")
                if (measCharV[4] == "c") exp2V <- c("garp", "harr")
                if (measCharV[4] == "d") exp2V <- c("bard", "capp", "fis", "list")
                if (measCharV[4] == "e") exp2V <- c("capp", "fis", "garp", "harr")
                if (measCharV[4] == "f") exp2V <- c("bard", "list")
                if (measCharV[4] == "g") exp2V <- c("capp", "garp", "harr")
            }


            if (measCharV[2]=="F") {
                bic <- 0
                llMax <- 0
                llMin <- 0
                actMin <- 0
                for (exp2 in exp2V) {
                    if (exp == exp2) next
                    
                    fname <- getAdeqFileName (exp2, model, MixMode, Moderate)
                    if (!file.exists(fname)) next
                    load (fname)
                    
                    x1 <- getAdeqItem (fname, "BIC", adeqLoc=adeq)
                    x2 <- getAdeqItem (fname, "LLmax", adeqLoc=adeq)
                    x3 <- getAdeqItem (fname, "LLmin", adeqLoc=adeq)
                    
                    bic <- bic + abs(x1) - abs(x2)
                    
                    if (measCharV[1] != "C") {
                        llMax <- llMax + abs(x2) - MakeLLNormalization * abs(x2)
                        llMin <- llMin + abs(x3) - MakeLLNormalization * abs(x2)
                    } else {
                        llMax <- llMax + abs(x1) - abs(x1)
                        llMin <- llMin + abs(x3) - abs(x1)
                    }
                }
            } else {
                bic <- getAdeqItem (fname, "BIC", adeqLoc=adeq)
            
                if (measCharV[1] == "C" && measCharV[2] == "B") {
                    if (scale=="LL") llMax <- bic
                    else {
                        llMax <- 0
                        llMin <- llMin - bic
                    }
                }
                else {
                    llMax <- llMax  # * length(setdiff (exp2V, exp))
                    llMin <- llMin  # * length(setdiff (exp2V, exp))
                }

                m <- m / length(setdiff (exp2V, exp))
            }

            if (measCharV[1] == "C" && measCharV[2] == "F") m <- m - abs(bic)
            else if (measCharV[1] == "C" && measCharV[2] == "B") m <- m
            else m <- m + (!MakeLLNormalization) * abs(llMax)
            
        } else  m <- m - MakeLLNormalization * abs(llMax)

        llMin <- abs(llMin) - MakeLLNormalization * abs(llMax)
        llMax <- abs(llMax) - MakeLLNormalization * abs(llMax)

    }

    if (scale == "R2") {
        m <- (llMin - m) / (llMin - llMax)
        llMax <- 1
        llMin <- 0
    }
#    if (measCharV[1] == "C") {  llMin <- NA ; llMax <- NA   }
        

    return (list(llMax=llMax, llMin=llMin, meas=m))
}

#############################################################################################################

absLL <- function (x) if (sum(x)>0) return (x) else return (-x)

loadLLV <- function (fname, measure, exp="", adeqLoc=NULL, SkipBICNormalization=FALSE) { # does not deal with pooled

    if (!file.exists(fname)) return (-1)
    if (is.null(adeqLoc)) load (fname) else adeq <- adeqLoc

#    cat (fname, "->", measure, "\n")

    vName <- paste (measure, "Ind", sep="")
    llV <- adeq[[vName]]
    
    measCharV <- strsplit (measure, split="")[[1]]
    meas12 <- paste (measCharV[1:2], collapse="")

    exp2V <- c("bard", "capp", "fis", "garp", "harr", "list")
    if (length(measCharV)==4) {
        if (measCharV[4] == "a") exp2V <- c("fis", "garp", "harr")
        if (measCharV[4] == "b") exp2V <- c("bard", "capp", "list")
        if (measCharV[4] == "c") exp2V <- c("garp", "harr")
        if (measCharV[4] == "d") exp2V <- c("bard", "capp", "fis", "list")
        if (measCharV[4] == "e") exp2V <- c("capp", "fis", "garp", "harr")
        if (measCharV[4] == "f") exp2V <- c("bard", "list")
        if (measCharV[4] == "g") exp2V <- c("capp", "garp", "harr")
    }
    expr2V <- setdiff (exp2V, exp)
    if (meas12=="PB") predFac <- length(expr2V) else predFac <- 1

    if (!is.null(llV)) {
        if (sum(is.infinite(llV)) + sum(is.nan(llV)) == length(llV)) { llV <- rep (400, length(llV)) }
        if (sum(is.nan(llV))>0) llV[is.nan(llV)] <- max (llV[!is.infinite(llV)], na.rm=TRUE)
        if (sum(is.infinite(llV))>0) llV[is.infinite(llV)] <- max (llV[!is.infinite(llV)])
        
        if (sum(llV)==0) return (-1)
        return (absLL(llV) / predFac)
    }

    if (SkipBICNormalization) {
        relPWa <- 0
        relBIC <- 0
        absBIC <- 0

    } else {
        relPWa <- loadLLV (fname, "PWa", adeqLoc=adeq) - loadLLV (fname, "LLmax", adeqLoc=adeq)
        relBIC <- loadLLV (fname, "BIC", adeqLoc=adeq) - loadLLV (fname, "LLmax", adeqLoc=adeq)
        absBIC <- 0
    }
    
    if (measure == "CW") return (loadLLV (fname, "PW", adeqLoc=adeq) - relPWa)
    if (measure == "CE") return (loadLLV (fname, "EW", adeqLoc=adeq) - relBIC)
    
    origMeasure <- paste (c("P", measCharV[2:length(measCharV)]), collapse="")
    
    if (measCharV[2] == "F") return (loadLLV (fname, origMeasure, exp=exp, adeqLoc=adeq) - absBIC)


    if (exp=="") YYY
        
    if (measCharV[2] == "B") {
        v1 <- loadLLV (fname, origMeasure, exp=exp, adeqLoc=adeq)
        ret <- (v1 - predFac*relBIC) / predFac
        
        return (ret)
    }

    return (-1)
}

loadAdeqIndivLL <- function (exp, model, MixMode, Moderate, measure, SkipBICNormalization=FALSE, SkipLLNormalization=FALSE) {
    # Loads the vector of indiv-ll, possibly pooled
    
    if (exp == "pool") exp2V <- c("bard", "capp", "fis", "garp", "harr", "list")           # All
    if (exp == "pool2") exp2V <- c("bard", "capp", "garp", "harr", "list")                 # Numerical
    if (exp == "pool3") exp2V <- c("capp", "fis", "garp", "harr")                          # All DG
    if (exp == "pool4") exp2V <- c("fis", "garp", "harr")                                  # Rich DG
    if (exp == "pool5") exp2V <- c("capp", "garp", "harr")                                 # Numerical DG
    if (exp == "pool6") exp2V <- c("bard", "list")                                         # (Numerical) TG
    if (exp == "pool7") exp2V <- c("garp", "harr")                                         # Standard DG
    if (exp == "pool8") exp2V <- c("bard", "capp", "list")                                 # Entitlement DG

    measCharV <- strsplit (measure, split="")[[1]]
        
    if (exp %in% c("pool", "pool2", "pool3", "pool4", "pool5", "pool6", "pool7", "pool8")) {

        llV <- c()
        for (exp2 in exp2V) {
            fname <- getAdeqFileName (exp2, model, MixMode, Moderate)
            if (!file.exists(fname)) next
#            load (fname)

            lastLoadLLV <<- loadAdeqIndivLL (exp2, model, MixMode, Moderate, measure, SkipBICNormalization=SkipBICNormalization, SkipLLNormalization=SkipLLNormalization)

            if (length(llV) != length(lastLoadLLV))  llV <- c(llV, pmin(1e5, lastLoadLLV))         # paste (measCharV[1:2], collapse="") != "CF" || exp2==exp2V[1]
            else llV <- llV + pmin(1e5, lastLoadLLV)

#            cat (exp2, ": length", length(lastLoadLLV), "=>", length(llV), "\tLL=", sum(lastLoadLLV), "=>", sum (llV), "\n")
            
        }

    } else {
        lastFName <<- fname <- getAdeqFileName (exp, model, MixMode, Moderate)
        if (!file.exists(fname)) return (-1)
        load (fname)

        lastFname <<- fname
        lastAdeq <<- adeq
        
        if (measCharV[2] != "F" && measCharV[2] != "B") {
            if (!SkipLLNormalization) llV <- pmin(1e5, loadLLV (fname, measure, exp, adeqLoc=adeq)) - pmin(1e5, loadLLV (fname, "LLmax", adeqLoc=adeq))
            else llV <- pmin(1e5, loadLLV (fname, measure, exp, adeqLoc=adeq))
            
        } else if (measCharV[2] == "F") {

            lastLLV <<- llV <- pmin(1e5, loadLLV (fname, measure, exp, adeqLoc=adeq))

            exp2V <- c("capp", "garp", "harr", "fis", "bard", "list")
            
            if (measCharV[2]=="F" && length(measCharV)==4) {
                if (measCharV[4] == "a") exp2V <- setdiff (exp2V, c("bard", "capp", "list"))
                if (measCharV[4] == "b") exp2V <- setdiff (exp2V, c("fis", "garp", "harr"))
                if (measCharV[4] == "c") exp2V <- setdiff (exp2V, c("bard", "capp", "fis", "list"))
                if (measCharV[4] == "d") exp2V <- setdiff (exp2V, c("garp", "harr"))
                if (measCharV[4] == "e") exp2V <- setdiff (exp2V, c("bard", "list"))
                if (measCharV[4] == "f") exp2V <- setdiff (exp2V, c("capp", "fis", "garp", "harr"))
                if (measCharV[4] == "g") exp2V <- setdiff (exp2V, c("bard", "fis", "list"))
            }

            bicLL <- c()
            llmaxLL <- c()
            for (exp2 in exp2V) {
                if (exp == exp2) next
                
                fname <- getAdeqFileName (exp2, model, MixMode, Moderate)
                if (!file.exists(fname)) next
                load (fname)
                
                lastX1 <<- x1 <- loadLLV (fname, "BIC", adeqLoc=adeq)
                lastX2 <<- x2 <- loadLLV (fname, "LLmax", adeqLoc=adeq)

#                cat (exp2, "=>", sum(absLL(x1) - absLL(x2)), " vs ", getAdeqItem (fname, "BIC")-getAdeqItem (fname, "LLmax"), "\n")
                
                bicLL <- c(bicLL, x1 - x2)
                llmaxLL <- c(llmaxLL, x2)
            }

            if (paste (measCharV[1:2], collapse="") == "CF" && !SkipBICNormalization) {
                if (length(llV) == length(bicLL)) llV <- llV - bicLL
                else if (length(llV) == length(bicLL)+1) llV <- llV[2:length(llV)] - bicLL
                else {
                    llV <- llV
                    lastLLV <<- llV
                    lastBicLL <<- bicLL
                    # XXX
                }
                
            } else if (SkipBICNormalization || SkipLLNormalization) {
                llV <- llV + llmaxLL
            }
            
            lastBicLL <<- bicLL
            
        } else { # measCharV[2] == "B"
            
            llV <- pmin(1e5, loadLLV (fname, measure, exp, adeqLoc=adeq, SkipBICNormalization))

            if (SkipBICNormalization || SkipLLNormalization) {
                llmaxV <- pmin(1e5, loadLLV (fname, measure="LLmax", exp, adeqLoc=adeq, SkipBICNormalization))
                llV <- llV + llmaxV
            }
        }
    }
    
    lastLLV <<- llV
    
    return (llV)    
}


makeLLtest <- function (model1, model2, measure, Nested=FALSE) {
#    cat ("makeLLtest:", model1, "vs", model2, "=>", measure, "\n")

    ll1 <- loadAdeqIndivLL (exp=model1[1], model=model1[2], MixMode=model1[3], Moderate=model1[4], measure)
    ll2 <- loadAdeqIndivLL (exp=model2[1], model=model2[2], MixMode=model2[3], Moderate=model2[4], measure)
    
    return (testLLs (LLs1=ll1, LLs2=ll2, ap1=0, ap2=0, isNested=Nested))
}


makeUniLLtest <- function (model1, measure, altMod="BIC") {
    cat ("makeUniLLtest:", model1, "=>", measure, altMod, "\n")

    if (altMod=="BIC") {
        ll1 <- loadAdeqIndivLL (exp=model1[1], model=model1[2], MixMode=model1[3], Moderate=model1[4], measure, SkipBICNormalization=TRUE)
        ll2 <- loadAdeqIndivLL (exp=model1[1], model=model1[2], MixMode=model1[3], Moderate=model1[4], measure)
        ll2 <- ll1-ll2
        
    } else {
        ll1 <- loadAdeqIndivLL (exp=model1[1], model=model1[2], MixMode=model1[3], Moderate=model1[4], measure)
        ll2 <- loadAdeqIndivLL (exp=model1[1], model=model1[2], MixMode=model1[3], Moderate=model1[4], measure, SkipLLNormalization=TRUE)
        ll1 <- ll1+ll2

#        print (c(length(ll1), length(ll2)))

    }
    
    pv <- NonNestLR (LLs1=ll1, LLs2=ll2, ap1=0, ap2=0)

    lastRet <<- ret <- list (meas=sum(ll1-ll2), pVal=pv, stars="")
    
    if (abs(ret$pVal)<0.005) ret$stars <- "$^{**}$"
    else if (abs(ret$pVal)<0.05) ret$stars <- "$^{*}$"
    
#    if (model == "prom") XXX
    
    return (ret)

}

#############################################################################################################


loadMomV <- function (fname, persp, measure) { # does not deal with pooled

    if (measure == "mean") num <- 1
    if (measure == "var") num <- 2
    if (measure == "entr") num <- 3
    if (measure == "sh0") num <- 4
    if (measure == "sh50") num <- 5
    if (measure == "sExpEntr") num <- 6
    if (measure == "min90") num <- 7
    if (measure == "shPos") num <- 8

    if (!file.exists(fname)) return (-1)
    load (fname)
    lastFName <<- fname

    if (persp == "In") persp <- "LL"

    matName <- paste (persp, "Mom", sep="")
    mat <- adeq[[matName]]
    if (is.null(mat)) return (-1)

    if (dim(mat)[2] >= num) momV <- mat[,num]
    else momV <- mat[,1] * 0

    return (momV)
}

loadAdeqMoments <- function (exp, model, MixMode, Moderate, persp, measure) {
    # persp in c(LLmax, LLmin, LL, BIC, PW, PWa, EW)
    # measure in c("mean", "var", "entr", "sh0", "sh50")
    # Loads the vector of moments across treatments, possibly pooled
    
    if (exp %in% c("pool", "pool2", "pool5", "pool7")) {
        
        if (exp == "pool") exp2V <- c("bard", "capp", "fis", "garp", "harr", "list")           # All
        if (exp == "pool2") exp2V <- c("bard", "capp", "garp", "harr", "list")                 # Numerical
        if (exp == "pool3") exp2V <- c("capp", "fis", "garp", "harr")                          # All DG
        if (exp == "pool4") exp2V <- c("fis", "garp", "harr")                                  # Rich DG
        if (exp == "pool5") exp2V <- c("capp", "garp", "harr")                                 # Numerical DG
        if (exp == "pool6") exp2V <- c("bard", "list")                                         # (Numerical) TG
        if (exp == "pool7") exp2V <- c("garp", "harr")                                         # Standard DG
        if (exp == "pool8") exp2V <- c("bard", "capp", "list")                                 # Entitlement DG

        momV <- c()
        for (exp2 in exp2V) {
            fname <- getAdeqFileName (exp2, model, MixMode, Moderate)
            if (!file.exists(fname)) next

            newMomV <- loadMomV (fname, persp, measure)
            momV <- c(momV, newMomV)
#            cat ("Pool - ", measure, ":", fname, ":", mean(newMomV), "=>", mean(momV), "\n")
            if (is.na (mean(newMomV))) cat (newMomV, "\n")
        }

    } else {
        fname <- getAdeqFileName (exp, model, MixMode, Moderate)

        momV <- loadMomV (fname, persp, measure)
    }

    lastFName <<- fname
    lastMomV <<- momV
    
    return (momV)
}

loadAdeqMAD <- function (exp, model, MixMode, Moderate, persp, measure) {
    # Loads the vector of mad from predicted moment to observed moment, possibly pooled

    predMomV <- loadAdeqMoments (exp, model, MixMode, Moderate, persp, measure)
    actMomV <- loadAdeqMoments (exp, model, MixMode, Moderate, "LLmax", measure)
    
#    madV <- abs (predMomV - actMomV)
    madV <- predMomV

    return (madV)
}

makeMADTest <- function (model1, model2, persp, measure, Nested=FALSE) {

    mad1 <- loadAdeqMAD (exp=model1[1], model=model1[2], MixMode=model1[3], Moderate=model1[4], persp, measure)
    mad2 <- loadAdeqMAD (exp=model2[1], model=model2[2], MixMode=model2[3], Moderate=model2[4], persp, measure)

    #XXX: make test of mad1-mad2, store result in pVal, set relSign
    
    return (testLLs (LLs1=mad1, LLs2=mad2, ap1=0, ap2=0, isNested=Nested))
}

makeMomentTest <- function (exp, model, MixMode, Moderate, persp, measure) {
    # tests if prediction is biased, i.e. mean deviation is positive or negative
    
    predMomV <- loadAdeqMoments (exp, model, MixMode, Moderate, persp, measure)
    actMomV <- loadAdeqMoments (exp, model, MixMode, Moderate, "LLmax", measure)
    madV <- abs (predMomV - actMomV)

    lastActMomV <<- actMomV
    lastPredMomV <<- predMomV
    
    #XXX: test madV and store result in pVal

    ret <- testLLs (LLs1=actMomV, LLs2=predMomV, ap1=0, ap2=0, isNested=FALSE)

    ret$predMom <- mean(predMomV, na.rm=TRUE)
    ret$actMom <- mean(actMomV, na.rm=TRUE)
    
    ret$stars <- ""
    if (abs(ret$pVal)<0.005) ret$stars <- "$^{**}$"
    else if (abs(ret$pVal)<0.05) ret$stars <- "$^{*}$"
    
    return (ret)
}

#############################################################################################################


getRdataFileName <- function (exp, model, MixMode, Moderate) {
    # exp in c("bard", "capp", "fis", "garp", "harr", "list")
    # model in c("logit", "ole", "palm2", "palm3", "prom")
    # MixMode in c(1,3,4,5),   Moderate in 0:2

    fn1 <- paste ("../", exp, "-mix-", model, "/RData/", sep="")

    if (Moderate==0) num <- c("", 2, 3, 5, 7)[as.double(MixMode)]
    if (Moderate==1) num <- c(9, 0, 4, 6, 8)[as.double(MixMode)]
    if (Moderate==2) num <- c(10, 0, 11, 12, 13)[as.double(MixMode)]

    exp2 <- exp
    if (exp2 == "garp") exp2 <- "opt"
    
    fn2 <- paste (exp2, "-mix", num, "-", model, ".Rdata.res1", sep="")

    fn <- paste (fn1, fn2, sep="")
    
    return (fn)
}

DiagInvMatr <- function (matr, estim) {
  degen <- c()
  FL <- 1
  while (FL) {
    degenOld <- degen
    max <- 0
    for (k in 1:nrow(matr)) 
      if (!(k %in% degen) && sum(abs(matr[k,]))>max) max <- sum(abs(matr[k,]))
    for (k in 1:nrow(matr)) 
      if (sum(abs(matr[k,]))<1e-5) { degen <- c(degen,k) }
    
    matr[,degen] <- 0
    matr[degen,] <- 0
    diag(matr)[degen] <- 1
    serr <- try (diag(solve (matr)), TRUE)
    if (!inherits(serr, "try-error")) break

    for (k in 1:nrow(matr)) 
      if (sum(abs(matr[k,]))==max) { degen <- c(degen,k) }
    
    matr[,degen] <- 0
    matr[degen,] <- 0
    diag(matr)[degen] <- 1

#    cat ("Degen: ", degen, "\n")
    FL <- ifelse (length(degen) > length(degenOld), 1, 0)
  }
  diag(matr)[degen] <- 1
  serr <- try (diag(solve (matr)), TRUE)
  if (inherits(serr, "try-error")) serr <- rep(NaN, length(estim))
  serr <- sqrt(abs(serr))
  serr[degen] <- NaN
  return (serr)
}


loadParamEstimate <- function (exp, model, MixMode, Moderate) {
    # load param-estimate for type=1 and drop type weight ("share")

    fname <- getRdataFileName (exp, model, MixMode, Moderate)    
#    cat (fname, "\n")
    if (!file.exists(fname)) { lastStandErr <<- 0 ; return (rep(-1, 8)) }

    load (fname)
    p <- res$estimate[2:length(res$estimate)]

    lastRes <<- res

    if (!is.null(res$se)) {
        se <- res$se
        lastStandErr <<- se[2:length(se)]
    } else {
        se <- DiagInvMatr (res$hessian, res$estimate)
        lastStandErr <<- 0
    }

    lastStandErr[is.na(lastStandErr)] <<- 100
    lastStandErr <<- pmin (1e2, lastStandErr)

    return (p)
}

########################################################################


# Are utility scales adaptative?
# normalize-mix4-logit-BIC-pool

if (1 %in% TablesDoBeDone)
for (MixMode in c(1,3,4,5))
    for (model in c("logit", "ole", "palm2", "palm3", "prom"))
        for (exp in c("pool", "pool3", "pool4", "pool5", "pool7", "bard", "capp", "fis", "garp", "harr", "list"))
            for (scale in c("LL", "L0", "R2"))
            for (measure in c("BIC", "PW", "EW", "PB1", "PB2", "PB1e", "PB2e", "PB3e", "PB3g",
                              "PB1a", "PB2a", "PB3a", "PB3c", "CB1a", "CB2a", "CB3a", "CB3c", "PF1", "PF1e")) {
            
                anzDigits <- ifelse (scale=="R2", 3,1)
                fSuff <- ""
                if (scale!="LL") fSuff <- paste ("-", scale, sep="")
                
                tabFile <- paste ("tex/normalize-mix", MixMode, "-", model, "-", measure, "-", exp, fSuff, ".tex", sep="")
                cat (tabFile, "\n")

                texOut <- c()
                prevMode <- -1
                for (Moderate in c(0,1,2)) {
                    m <- loadAdeqMeasure (exp, model, MixMode, Moderate, measure, scale)

                    if (prevMode<0) texOut <- c(texOut, round(m$meas, digits=anzDigits))
                    else {
                        llTest <- makeLLtest (c(exp, model, MixMode, prevMode), c(exp, model, MixMode, Moderate),
                                               measure, Nested=FALSE)
                        
                        texOut <- c(texOut, llTest$relSign, round(m$meas, digits=anzDigits))
                    }

                    prevMode <- Moderate
                }

                zz <- file (tabFile, "w")
                cat (paste (texOut, collapse=" & "), file=zz)
                close (zz)
            
            }



########################################################################

# How do subjects differ?
# heterog-mod1-logit-BIC-pool

if (2 %in% TablesDoBeDone)
for (Moderate in c(0,2,1))
    for (model in c("logit", "ole", "palm2", "palm3", "prom"))
        for (exp in c("pool", "pool3", "pool4", "pool5", "pool7", "bard", "capp", "fis", "garp", "harr", "list"))
            for (scale in c("LL", "L0", "R2"))
            for (measure in c("BIC", "PW", "EW", "PB1", "PB2", "PB1e", "PB2e", "PB1a", "PB2a", "PB3a", "PB3c", "PB3e", "PF1", "PF1e", "PB3g")) {
            
                anzDigits <- ifelse (scale=="R2", 3,1)
                fSuff <- ""
                if (scale!="LL") fSuff <- paste ("-", scale, sep="")
                
                tabFile <- paste ("tex/heterog-mod", Moderate, "-", model, "-", measure, "-", exp, fSuff, ".tex", sep="")
                cat (tabFile, "\n")

                texOut <- c()
                prevMode <- -1
                for (MixMode in c(1,3,4,5)) {
                    m <- loadAdeqMeasure (exp, model, MixMode, Moderate, measure, scale)

                    if (prevMode<0) texOut <- c(texOut, round(m$meas, digits=anzDigits))
                    else {
                        llTest <- makeLLtest (c(exp, model, prevMode, Moderate), c(exp, model, MixMode, Moderate),
                                               measure, Nested=(measure=="BIC"))
                        
                        texOut <- c(texOut, llTest$relSign, round(m$meas, digits=anzDigits))
                    }
                    
                    prevMode <- MixMode
                }

                zz <- file (tabFile, "w")
                cat (paste (texOut, collapse=" & "), file=zz)
                close (zz)
            
            }




########################################################################

# How do subjects choose transfers?
# choice-mix4-mod1-BIC-pool

correctRelSign <- function (s, scale) {
    if (scale!="R2") return (s)

    if (s=="$<$") return ("$>$")
    if (s=="$>$") return ("$<$")
    if (s=="$\\ll$") return ("$\\gg$")
    if (s=="$\\gg$") return ("$\\ll$")
    return (s)
}

if (3 %in% TablesDoBeDone)
for (Moderate in c(0,2,1))
    for (MixMode in c(3,1,4,5))
        for (exp in c("pool3", "pool5", "pool7", "capp", "fis", "garp", "harr",
                      "pool", "pool2", "pool4", "pool6", "pool8", "bard", "list"))
            for (scale in c("L0", "R2", "LL"))
            for (measure in c("BIC",
                              "PB1a", "CB1a", "PF1", "CF1", "PF1e", "CF1e", "PF1a", "CF1a",
                              "PB2a", "CB2a", "PF2", "CF2", "PF2e", "CF2e", "PF2a", "CF2a",
                              "PB3a", "CB3a", "PB3c", "CB3c", "PB3g", "CB3g", "PF3g", "CF3g",
                              "PW", "EW", "PB1", "PB1e", "PB2", "PB2e", "CB1e", "CB2e")) {

                # Full information table
            
                anzDigits <- ifelse (scale=="R2", 3,1)
                fSuff <- ""
                if (scale!="LL") fSuff <- paste ("-", scale, sep="")
                
                tabFile <- paste ("tex/choice-mix", MixMode, "-mod", Moderate, "-", measure, "-", exp, fSuff, ".tex", sep="")
                cat (tabFile, "\n")
                
                mProm <- loadAdeqMeasure (exp, "prom", MixMode, Moderate, measure, scale)

                texOut <- c()
                prevMode <- ""
                ct <- 0
                for (model in c("logit", "prom", "ole", "palm3", "logit")) {
                    ct <- ct+1
                    
                    m <- loadAdeqMeasure (exp, model, MixMode, Moderate, measure, scale)
                    
                    measCharV <- strsplit (measure, split="")[[1]]
                    if (measCharV[1]=="C") {
                        if (measCharV[1]=="C") ut <- makeUniLLtest (c(exp, model, MixMode, Moderate), measure)
                        else ut <- makeUniLLtest (c(exp, model, MixMode, Moderate), measure, altMod="LLmax")
                            
                        stars <- ut$stars
                    } else stars <- ""

                    if (prevMode=="") {
                        texOut <- c(texOut, round(mProm$llMax, digits=1), "", round(mProm$llMin, digits=1), "",
                                    paste (round(m$meas, digits=anzDigits), stars, sep=""))
                    } else {
                        llTest <- makeLLtest (c(exp, prevMode, MixMode, Moderate), c(exp, model, MixMode, Moderate),
                                               measure, Nested=(ct<=3 && measure=="BIC"))
                        
                        texOut <- c(texOut, correctRelSign (llTest$relSign, scale),
                                    paste (round(m$meas, digits=anzDigits), stars, sep=""))
                    }
                    
                    prevMode <- model
                }

                zz <- file (tabFile, "w")
                cat (paste (texOut, collapse=" & "), file=zz)
                close (zz)
                

                
                # Shorter table
            
                tabFile <- paste ("tex/choice-mix", MixMode, "-mod", Moderate, "-", measure, "-", exp, fSuff, ".B.tex", sep="")
                cat (tabFile, "\n")

                texOut <- c()
                prevMode <- ""
                ct <- 0
                for (model in c("prom", "ole", "palm3", "logit")) {
                    ct <- ct+1
                    
                    m <- loadAdeqMeasure (exp, model, MixMode, Moderate, measure, scale)

                    measCharV <- strsplit (measure, split="")[[1]]
                    if (measCharV[1]=="C") {
                        if (measCharV[1]=="C") ut <- makeUniLLtest (c(exp, model, MixMode, Moderate), measure)
                        else ut <- makeUniLLtest (c(exp, model, MixMode, Moderate), measure, altMod="LLmax")
                            
                        stars <- ut$stars
                    } else stars <- ""
                    
                    if (prevMode=="") {
                        texOut <- c(texOut, round(mProm$llMax, digits=1), "", round(mProm$llMin, digits=1), "",
                                    paste (round(m$meas, digits=anzDigits), stars, sep=""))
                    } else {
                        llTest <- makeLLtest (c(exp, prevMode, MixMode, Moderate), c(exp, model, MixMode, Moderate),
                                               measure, Nested=(ct<=3 && measure=="BIC"))
                        
                        texOut <- c(texOut, correctRelSign (llTest$relSign, scale),
                                    paste (round(m$meas, digits=anzDigits), stars, sep=""))
                    }
                    
                    prevMode <- model
                }

                zz <- file (tabFile, "w")
                cat (paste (texOut, collapse=" & "), file=zz)
                close (zz)

                zz <- file (paste(tabFile, 2, sep=""), "w")
                cat (paste (texOut[5:length(texOut)], collapse=" & "), file=zz)
                close (zz)

            }


########################################################################

# Are stylized facts reproduced?
# styl-mean-In-mix4-mod1-pool

if (4 %in% TablesDoBeDone)
for (Moderate in c(0,2,1))
    for (MixMode in c(1,3,4,5))
        for (exp in c("pool", "pool5", "pool7", "bard", "garp", "harr", "list", "capp"))
            for (persp in c("In", "PW", "PB1a", "PB2a", "PB3a", "PB3c", "PF1e", "PF1f"))
                for (measure in c("entr", "sh0", "sh50", "sExpEntr", "min90", "shPos")) { # "mean", "var", 

                    # Full information table
                    
                    tabFile <- paste ("tex/styl-mix", MixMode, "-mod", Moderate, "-", measure, "-", persp, "-", exp, ".tex", sep="")
                    cat (tabFile, "\n")
                    
                    texOut <- c()
                    prevMode <- ""
                    for (model in c("logit", "ole", "palm3", "prom", "palm2")) {
                        m <- makeMomentTest (exp, model, MixMode, Moderate, persp, measure)
                        
                        if (prevMode=="") texOut <- c(texOut, round (m$actMom, digits=ifelse(m$actMom<10, 2, 1)), "")
                        else {
                            madTest <- makeMADTest (c(exp, prevMode, MixMode, Moderate), c(exp, model, MixMode, Moderate),
                                                   persp, measure, Nested=FALSE)
                            
                            texOut <- c(texOut, madTest$relSign)
#                            texOut <- c(texOut, "")
                        }

                        texOut <- c(texOut, paste(round(m$predMom, digits=ifelse(m$predMom<10, 2, 1)), m$stars, sep=""))
                        
                        prevMode <- model

                        # Moments by treatment
                        tabFile2 <- paste ("tex/styl-tr-", model, "-mix", MixMode, "-mod", Moderate, "-", measure, "-", persp, "-", exp, ".tex", sep="")
                        zz2 <- file (tabFile2, "w")
                        cat (paste (round (lastPredMomV, digits=2), collapse=" & "), file=zz2)
                        close (zz2)
                    }

                    # Save information on average across treatments
                    zz <- file (tabFile, "w")
                    cat (paste (texOut, collapse=" & "), file=zz)
                    close (zz)


                    # Actual moments by treatment
                    tabFile2 <- paste ("tex/styl-tr-", "LLmax", "-mix", MixMode, "-mod", Moderate, "-", measure, "-", persp, "-", exp, ".tex", sep="")
                    zz2 <- file (tabFile2, "w")
                    cat (paste (round (lastActMomV, digits=2), collapse=" & "), file=zz2)
                    close (zz2)


                    
                    
                    # Short table
                    
                    tabFile <- paste ("tex/styl-mix", MixMode, "-mod", Moderate, "-", measure, "-", persp, "-", exp, ".B.tex", sep="")
                    cat (tabFile, "\n")
                    
                    texOut <- c()
                    prevMode <- ""
                    for (model in c("prom", "ole", "palm3", "logit")) {
                        m <- makeMomentTest (exp, model, MixMode, Moderate, persp, measure)
                        
                        if (prevMode=="") texOut <- c(texOut, round (m$actMom, digits=ifelse(m$actMom<10, 2, 1)), "")
                        else {
                            madTest <- makeMADTest (c(exp, prevMode, MixMode, Moderate), c(exp, model, MixMode, Moderate),
                                                   persp, measure, Nested=FALSE)
                            
                            texOut <- c(texOut, madTest$relSign)
#                            texOut <- c(texOut, "")
                        }

                        texOut <- c(texOut, paste(round(m$predMom, digits=ifelse(m$predMom<10, 2, 1)), m$stars, sep=""))
                        
                        prevMode <- model
                    }

                    zz <- file (tabFile, "w")
                    cat (paste (texOut, collapse=" & "), file=zz)
                    close (zz)

                    
                }


#####################################################################################################

# Are parameters robustly identified?
# pred-mix4-mod1-PB1-garp - YYY

if (length (setdiff (TablesDoBeDone, c(1:4, 8:20))) != 0)
for (Moderate in TablesDoBeDone-5) {
    if (Moderate<0 || Moderate>2) next
    
    for (MixMode in c(3,4,1,5))
        for (exp in c("pool3", "pool5", "pool7", "capp", "fis", "garp", "harr",
                      "pool", "pool2", "pool4", "pool6", "pool8", "bard", "list"))
#        for (exp in c("pool4", "garp", "harr"))
            for (scale in c("LL", "L0", "R2"))
            for (measure in c("PB1a", "CF1", "PF1", "PB1", "PB2", "PF2", "PB3", "PF3",
                              "PB1a", "PB2a", "PB3a", "PB3c", "PB1e", "PB2e", "PB3e", "PB1f", "PB2f", "PB3f", "PB3g",
                              "PF1e", "PF1f", "PF2e", "PF2f", "PF3e", "PF3f", "PF3g",
                              "CB1", "CF1", "CB2", "CF2", "CB3", "CF3",
                              "CB1a", "CB2a", "CB3a", "CB3c", "CB1e", "CB2e", "CB3e", "CB1f", "CB2f", "CB3f", "CB3g",
                              "CF1e", "CF2e", "CF3e", "CF1f", "CF2f", "CF3f", "CF3g"
                              )) {
                
                # Full information table

                anzDigits <- ifelse (scale=="R2", 3,1)
                fSuff <- ""
                if (scale!="LL") fSuff <- paste ("-", scale, sep="")
                
                tabFile <- paste ("tex/pred-mix", MixMode, "-mod", Moderate, "-", measure, "-", exp, fSuff, ".tex", sep="")
                cat (tabFile, "\n")

                texOut <- c()
                prevMode <- ""
                ct <- 0
                for (model in c("logit", "prom", "ole", "palm3", "logit")) {
                    ct <- ct+1
                    
                    m <- loadAdeqMeasure (exp, model, MixMode, Moderate, measure, scale)

                    measCharV <- strsplit (measure, split="")[[1]]
                    if (measCharV[1]=="C") {
                        if (measCharV[1]=="C") ut <- makeUniLLtest (c(exp, model, MixMode, Moderate), measure)
                        else ut <- makeUniLLtest (c(exp, model, MixMode, Moderate), measure, altMod="LLmax")
                            
                        stars <- ut$stars
                    } else stars <- ""

                    if (prevMode=="") {
                        texOut <- c(texOut, round(m$llMax, digits=1), "", round(m$llMin, digits=1), "",
                                    paste (round(m$meas, digits=anzDigits), stars, sep=""))
                    } else {
                        llTest <- makeLLtest (c(exp, prevMode, MixMode, Moderate), c(exp, model, MixMode, Moderate),
                                               measure, Nested=(ct<=3 && measure=="BIC"))
                        
                        texOut <- c(texOut, correctRelSign (llTest$relSign, scale),
                                    paste (round(m$meas, digits=anzDigits), stars, sep=""))
                    }
                    
                    prevMode <- model
                }

                zz <- file (tabFile, "w")
                cat (paste (texOut, collapse=" & "), file=zz)
                close (zz)
                
            
                # Short table
                
                tabFile <- paste ("tex/pred-mix", MixMode, "-mod", Moderate, "-", measure, "-", exp, fSuff, ".B.tex", sep="")
                cat (tabFile, "\n")

                texOut <- c()
                prevMode <- ""
                ct <- 0
                for (model in c("prom", "ole", "palm3", "logit")) {
                    ct <- ct+1
                    
                    m <- loadAdeqMeasure (exp, model, MixMode, Moderate, measure, scale)

                    measCharV <- strsplit (measure, split="")[[1]]
                    if (measCharV[1]=="C") {
                        if (measCharV[1]=="C") ut <- makeUniLLtest (c(exp, model, MixMode, Moderate), measure)
                        else ut <- makeUniLLtest (c(exp, model, MixMode, Moderate), measure, altMod="LLmax")
                            
                        stars <- ut$stars
                    } else stars <- ""

                    if (prevMode=="") {
                        texOut <- c(texOut, round(m$llMax, digits=1), "", round(m$llMin, digits=1), "",
                                    paste (round(m$meas, digits=anzDigits), stars, sep=""))
                    } else {
                        llTest <- makeLLtest (c(exp, prevMode, MixMode, Moderate), c(exp, model, MixMode, Moderate),
                                               measure, Nested=(ct<=3 && measure=="BIC"))
                        
                        texOut <- c(texOut, correctRelSign (llTest$relSign, scale),
                                    paste (round(m$meas, digits=anzDigits), stars, sep=""))
                    }
                    
                    prevMode <- model
                }

                zz <- file (tabFile, "w")
                cat (paste (texOut, collapse=" & "), file=zz)
                close (zz)
            
            }
}

#####################################################################################################

# Parameter estimates
# pars-mix4-mod1-prom-garp

if (8 %in% TablesDoBeDone) {

    # Full information table
    
    for (Moderate in c(0,2,1))
        for (MixMode in c(1,3,4,5))
            for (exp in c("bard", "capp", "fis", "garp", "harr", "list"))
                for (model in c("logit", "ole", "palm3", "prom", "palm2")) {
                    
                    tabFile <- paste ("tex/pars-mix", MixMode, "-mod", Moderate, "-", model, "-", exp, ".tex", sep="")
                    cat (tabFile, "\n")
                    
                    p <- loadParamEstimate (exp, model, MixMode, Moderate)
                    
                    if (sum(lastStandErr) != 0) {
                        t1 <- function (a)
                            paste ("\\un{(", round(lastStandErr[a], digits=3), ")}{", round(p[a], digits=2), "}& $-$ ")
                        t2 <- function (a,b)
                            paste ("\\un{(", round(lastStandErr[a], digits=3), ")}{", round(p[a], digits=2), "}",
                                   "& \\un{(", round(lastStandErr[b], digits=3), ")}{", round(p[b], digits=2), "}")
                    } else {
                        t1 <- function (a) paste ("$", round(p[a], digits=2), "$ & ")
                        t2 <- function (a,b) paste (round(p[a], digits=2), "&", round(p[b], digits=2))
                    }

                    if (model=="logit") {
                        if (MixMode==1) texOut <- c(t2(4,6), t2(5,7), t1(1))
                        if (MixMode==3) texOut <- c(t2(4,6), t2(5,7), t2(1,8))
                        if (MixMode==4) texOut <- c(t2(4,6), t2(5,7), t2(1,8))
                        if (MixMode==5) texOut <- c(t2(4,6), t2(5,7), t1(1))
                        
                    } else {
                        if (MixMode==1) texOut <- c(t2(4,6), t2(5,7), t1(1), t1(2))
                        if (MixMode==3) texOut <- c(t2(4,6), t2(5,7), t2(1,8), t1(2))
                        if (MixMode==4) texOut <- c(t2(4,6), t2(5,7), t2(1,8), t2(2,3))
                        if (MixMode==5) texOut <- c(t2(4,6), t2(5,7), t1(1), t2(2,3))
                        
                    }
                    
                    zz <- file (tabFile, "w")
                    cat (paste (texOut, collapse=" && "), file=zz)
                    close (zz)
                    
                }

    # Short table: Reports only degree of altruism
    
    for (Moderate in c(0,2,1))
        for (MixMode in c(1,3,4,5))
            for (measure in 1:2)
                for (exp in c("bard", "capp", "fis", "garp", "harr", "list")) {
                    tabFile <- paste ("tex/pars-mix", MixMode, "-mod", Moderate, "-", exp, ".B", measure, ".tex", sep="")
                    cat (tabFile, "\n")
                    
                    texOut <- c()
                    for (model in c("prom", "ole", "palm3", "logit")) {
                        
                        p <- loadParamEstimate (exp, model, MixMode, Moderate)

                        if (sum(lastStandErr) != 0) {
                            t2s <- function (a,b)
                               paste ("\\un{(", round(lastStandErr[a], digits=3), ")}{", round(p[a], digits=2), "}",
                                    "& \\un{(", round(lastStandErr[b], digits=3), ")}{", round(p[b], digits=2), "}")
                        } else {
                            t2s <- function (a,b)
                               paste (round(p[a], digits=2), " & ", round(p[b], digits=2))
                        }

                        if (measure==1) texOut <- c(texOut, t2s(4,6))
                        if (measure==2) texOut <- c(texOut, t2s(5,7))
                    }
                    
                    zz <- file (tabFile, "w")
                    cat (paste (texOut, collapse=" && "), file=zz)
                    close (zz)
                }
    
    # Short table: Reports only real means of preference parameters
    
    for (Moderate in c(0,2,1))
        for (MixMode in c(1,3,4,5))
            for (exp in c("bard", "capp", "fis", "garp", "harr", "list")) {
                tabFile <- paste ("tex/pars-mix", MixMode, "-mod", Moderate, "-", exp, ".C", ".tex", sep="")
                cat (tabFile, "\n")
                
                texOut <- c()
                for (model in c("prom", "ole", "palm3", "logit")) {
                    
                    p <- loadParamEstimate (exp, model, MixMode, Moderate)
                    
                    t2t <- function (a,b) {
                        if (pnorm((0.5-p[a])/p[b]) < 1e-2) return (0.5)
                        m <- p[a] + (dnorm((-0.5-p[a])/p[b]) - dnorm((0.5-p[a])/p[b])) / (pnorm((0.5-p[a])/p[b]) - pnorm((-0.5-p[a])/p[b])) * p[b]
                        
                        return (round (m, digits=2))
                    }

                    t2n <- function (a,b) return (round (p[a], digits=2))
                    
                    texOut <- c(texOut, paste (t2t(4,6), "&", t2n(5,7)))
                }
                
                zz <- file (tabFile, "w")
                cat (paste (texOut, collapse=" && "), file=zz)
                close (zz)
            }
}


#####################################################################################################

# Distribution of choices
# lev-distr-garp-1.tex

getAnzStrats <- function (exp, tr) {
    if (exp=="fis") AnzStrats <- 101
    else if (exp!="capp") AnzStrats <- (Up[tr] - Low[tr]) / Step[tr] + 1
    else {
        lineNr <- ((1:length(RefData$Up))[RefData$Treat==tr])[1]
        AnzStrats <- RefData$Up[lineNr] + 1
    }
    
    return (AnzStrats)
}

getEqSplitIndex <- function (exp, tr) {

    if (exp %in% c("garp", "harr")) {
        es <- (DictEndow[tr] - RecEndow[tr] - (1+Rate[tr]) * Low[tr]) / ((1+Rate[tr]) * Step[tr])
        es <- round (es + 1)

    } else if (exp %in% c("list", "bard")) {
        es <- (DictEndow[tr] - RecEndow[tr] - (1+Rate[tr]) * Low[tr]) / ((1+Rate[tr]) * Step[tr])
        es <- round (es + 1)
        
    } else if (exp=="capp") {
        as <- getAnzStrats (exp, tr)
        es <- 1 + (as-1) / 2
        
    } else if (exp=="fis") {
        es <- 50 + 1
    }

    return (es)
}

getRatioLabel <- function (exp, tr) {

    if (exp %in% c("garp", "harr")) {
        lab <- paste (Step[tr], ":", round(Step[tr]*Rate[tr]), sep="")

    } else if (exp %in% c("list", "bard")) {
        lab <- paste (1, ":", round(Rate[tr]), sep="")
        
    } else if (exp=="capp") {
        lab <- "1:1"
        
    } else if (exp=="fis") {
        ratioLab <- c("4:1", "3:1", "5:2", "2:1", "3:2", "1:1", "1:2", "1:3")
        lab <- ratioLab[tr]
    }

    return (lab)
}

getStratLabelV <- function (exp, tr) {

    AnzStrats <- getAnzStrats (exp, tr)
    
    if (exp %in% c("garp", "harr")) {
        transV <- 1:AnzStrats-1

    } else if (exp %in% c("list", "bard")) {
        transV <- (1:AnzStrats-1)*Step[tr] + Low[tr]
        
    } else if (exp=="capp") {
        transV <- (1:AnzStrats-1)
        
    } else if (exp=="fis") {
        transV <- (1:AnzStrats-1)
    }

    return (transV)
}

getEmpChoices <- function (exp, tr) { # returns vector of strategy index
    
    if (exp %in% c("garp", "harr")) {
        stratV <- data[,tr]
        NormStratV <- round((Up[tr] - Low[tr])/Step[tr]) - stratV + 1
        
    } else if (exp %in% c("list", "bard")) {
        NormStratV <- round((data[tr,1:AnzDat[tr]] - Low[tr])/Step[tr] + 1)        

    } else if (exp == "capp") {
        NormStratV <- RefData$NormPass[RefData$Treat==tr]+1
        
    } else if (exp=="fis") {
        ratioV <- c(4/1, 3/1, 5/2, 2/1, 3/2, 1/1, 1/2, 1/3)
        r <- ratioV[tr]
        inclV <- (RefData$Rate>r/1.1 & RefData$Rate<r*1.1)
        lastActChoicesFIS <<- RefData$NormPass[inclV]
        lastActMaxFis <<- RefData$Up[inclV]
#        cat (lastActChoicesFIS, "\n")
        NormStratV <- round(RefData$NormPass[inclV]/RefData$Up[inclV]*100) + 1
    }

    return (NormStratV)
}

getLevProm <- function (exp, tr) {
  if (exp=="fis") prLevs <- rep (0, 101)
  else if (exp!="capp") prLevs <- GetProminenceLevs (tr, subj=subj, FixPromZero=1)
  else {
    lineNr <- ((1:length(RefData$Up))[RefData$Treat==tr])[1]
    prLevs <- GetProminenceLevs (tr, subj=lineNr, FixPromZero=1)
  }
  
  return (prLevs)
}

getRelFreqs <- function (choiceV) {

    ret <- c()
    for (ch in sort (unique (choiceV)))
        ret <- c(ret, sum(choiceV==ch) / length(choiceV))

    return (ret)
}
    
CappFullStratSpace <- TRUE
DoNotLoadDLL <- TRUE

if (8 %in% TablesDoBeDone) {
    
    numChoicesV <- c()
    meanChoiceV <- c()
    numDiffChoicesV <- c()
    min90V <- c()
    expEntrV <- c()
    numOptMaxLevV <- c()
    lev1V <- c()
    relLevEqV <- c()
    freqEqSplitV <- c()
    
    levProbV2 <- c()
    levProbV1 <- c()
    levProbV2a <- c()
    levProbV1a <- c()
    levProbV2b <- c()
    levProbV1b <- c()
    
    for (exp in c("garp", "harr", "capp", "list", "bard", "fis")) {
        
        rm(list=ls(pattern="prLevs"))
        
        datFName <- paste ("../", exp, "-data.R", sep="")
        source (datFName)

        if (exp %in% c("garp", "harr")) inpFName <- "../garp-input.R"
        else if (exp %in% c("bard", "list")) inpFName <- "../bard-input.R"
        else if (exp %in% c("fis")) inpFName <- "../fis-input.R"
        else inpFName <- "../capp-input.R"

        source (inpFName)

        choiceDenomV <- rep (0, 13)
        choiceDenomCt <- rep (0, 13)

        if (exp=="fis") AnzTreat <- 8

        eESLabelV <- c()
        eESRelLevV <- c()
        eESRelFreqV <- c()
        eQ100V <- c()
        eQ90V <- c()
        eEntrV <- c()
        eExpEntrV <- c()
        eTopOptionsV <- c()
        eTopRelFreqV <- c()
        eOtherRelFreqV <- c()
        
        for (treat in 1:AnzTreat) {

            # Basic information on treatments

            stratLab <- getStratLabelV (exp, treat)
            choiceIndV <- getEmpChoices (exp, treat)
            relFreqs <- getRelFreqs (choiceIndV)
            relFreqs2 <- c(relFreqs, rep (0, length(stratLab) - length(relFreqs)))
            eqSplInd <- pmax (0, pmin (length(stratLab), getEqSplitIndex (exp, treat)))

            outV1 <- rep("", 4)
            outV2 <- rep("", 11)

            numChoices <- outV1[1] <- length (stratLab)
            numObs <- outV1[2] <- length (choiceIndV)
            ratioLab <- outV1[3] <- getRatioLabel (exp, treat)
            meanChoice <- outV1[4] <- round (mean (stratLab[choiceIndV]), digits=2)

            if (exp=="fis") {
                numChoices <- outV1[1] <- "NA"
                ratioLab <- outV1[3] <- paste ("$\\approx$", ratioLab)
            }
            
            tabFile <- paste ("tex/lev-distr-", exp, "-", treat, ".1.tex", sep="")
            zz <- file (tabFile, "w")
            cat (paste (outV1, collapse=" & "), file=zz)
            close (zz)


            # Information on entropy and prominence
            
            labEqSplit <- outV2[1] <- ifelse (exp=="fis", NA, stratLab[eqSplInd])
            freqEqSplit <- outV2[3] <- round (sum (choiceIndV==eqSplInd) / length (choiceIndV), digits=2)

            # How to measure the "concentration" on few options
            # sd of relative frequencies: Is not maximal if all is on one option
            # l2-Norm of rel freqs: About okay
            # entropy: reasonable alternative measure, but largely invariant as to whether choice focuses on 5 out of 10 options or 5 out of 100
            # e(entropy): Number of options fully considered
            # "concentration": how many options account for 50%, 75%, 90% of choices
            # How many options are more frequent than uniform randomization: Problem as with sd
            # gini: label/value of options matters, which is supposed to be left out here
            
            d <- length(stratLab)
            numDiffChoices <- outV2[5] <- length(relFreqs)
            cs <- cumsum(-sort (-relFreqs))
            min90 <- outV2[6] <- round (which.max(cs>=0.9) / 1, digits=2)

            absFreqs <- round (relFreqs2 * length(choiceIndV))
            entr <- entropy (absFreqs, method="MM")       # sum (-relFreqs * log(relFreqs))
            outV2[7] <- round (entr, digits=2)
            expEntr <- outV2[8] <- round (exp(entr), digits=2)
            
            l2 <- round ((sum(relFreqs2^2)^(1/2) * sqrt(d) - 1) / (sqrt(d) - 1), digits=2)

            sdMax <- sd (c(1,rep(0, d-1)))
            sdRF <- round (sd(relFreqs2)/sdMax, digits=2)
            
#            library (ineq)
#            gini <- round (ineq (stratLab[choiceIndV]-min(stratLab[choiceIndV]), type="Gini"), digits=2)

            if (exp=="fis") {
                highLev <- lastActChoicesFIS %% 10==0
                numOptMaxLev <- outV2[10] <- round (sum(floor((lastActMaxFis+0.5)/10)) / length(lastActMaxFis), digits=1)
                lev1 <- outV2[11] <- round (sum(highLev) / length(highLev), digits=2)
                lev2 <- outV2[12] <- round (sum(!highLev) / length(highLev), digits=2)

                relLevEq <- outV2[2] <- -1
            } else {
                levProm <- getLevProm (exp, treat)
                maxLev <- max (levProm)
                if (sum(levProm==maxLev)<3) maxLev <- maxLev-1
                if (sum(levProm==maxLev)<3) maxLev <- maxLev-1
                levProm[1] <- maxLev                     # here: "minimal" choice counts as one of the most prominent
                
                numOptMaxLev <- outV2[10] <- sum(levProm>=maxLev)
                lev1 <- outV2[11] <- round (sum(levProm[choiceIndV]>=maxLev) / length(choiceIndV), digits=2)
                lev2 <- outV2[12] <- round (sum(levProm[choiceIndV]<=maxLev-1) / length(choiceIndV), digits=2)

                relLevEq <- outV2[2] <- pmin (0, levProm[eqSplInd] - maxLev)
                                
#            lev3 <- outV[15] <- round (sum(levProm[choiceIndV]==maxLev-2) / length(choiceIndV), digits=2)
#            levOth <- outV[16] <- round (sum(levProm[choiceIndV]<=maxLev-3) / length(choiceIndV), digits=2)
            }

            elevProb2 <- mean(levProm[choiceIndV]>=2 & stratLab[choiceIndV]>4  & stratLab[choiceIndV]<16)
            elevProb1 <- mean(levProm[choiceIndV]<=1 & stratLab[choiceIndV]>4  & stratLab[choiceIndV]<16)

            myMax <- max(stratLab)
            myMax <- myMax + (myMax %% 10)-5   # myMax/2 - ((myMax/2) %% 10) + 4

            elevProb2a <- mean(levProm[choiceIndV]>=2 & stratLab[choiceIndV]>4  & stratLab[choiceIndV]<myMax)
            elevProb1a <- mean(levProm[choiceIndV]<=0 & stratLab[choiceIndV]>4  & stratLab[choiceIndV]<myMax)
            
            elevProb2b <- mean(levProm[choiceIndV]>=2 & stratLab[choiceIndV]>4  & stratLab[choiceIndV]<35)
            elevProb1b <- mean(levProm[choiceIndV]<=0 & stratLab[choiceIndV]>4  & stratLab[choiceIndV]<35)
            
            tabFile <- paste ("tex/lev-distr-", exp, "-", treat, ".2.tex", sep="")
            zz <- file (tabFile, "w")
            cat (paste (outV2, collapse=" & "), file=zz)
            close (zz)

            eESLabelV <- c(eESLabelV, outV2[1])
            eESRelLevV <- c(eESRelLevV, outV2[2])
            eESRelFreqV <- c(eESRelFreqV, outV2[3])
            eQ100V <- c(eQ100V, outV2[5])
            eQ90V <- c(eQ90V, outV2[6])
            eEntrV <- c(eEntrV, outV2[7])
            eExpEntrV <- c(eExpEntrV, outV2[8])
            eTopOptionsV <- c(eTopOptionsV, outV2[10])
            eTopRelFreqV <- c(eTopRelFreqV, outV2[11])
            eOtherRelFreqV <- c(eOtherRelFreqV, outV2[12])


            if (exp!="fis") {
                numChoicesV <- c(numChoicesV, numChoices)
                meanChoiceV <- c(meanChoiceV, meanChoice)
                numDiffChoicesV <- c(numDiffChoicesV, numDiffChoices)
                min90V <- c(min90V, min90)
                expEntrV <- c(expEntrV, expEntr)
                numOptMaxLevV <- c(numOptMaxLevV, numOptMaxLev)
                lev1V <- c(lev1V, lev1)
                relLevEqV <- c(relLevEqV, relLevEq)
                freqEqSplitV <- c(freqEqSplitV, freqEqSplit)
                
                levProbV1 <- c(levProbV1, elevProb1)
                levProbV2 <- c(levProbV2, elevProb2)
                levProbV1a <- c(levProbV1a, elevProb1a)
                levProbV2a <- c(levProbV2a, elevProb2a)
                levProbV1b <- c(levProbV1b, elevProb1b)
                levProbV2b <- c(levProbV2b, elevProb2b)
            }


            # Basic listing on distribution of choices
            
            choiceLabV <- stratLab[choiceIndV]
            if (exp=="fis") choiceLabV <- choiceLabV/10
            choiceFL <- rep (TRUE, length(choiceLabV))

            outV3 <- c(min(stratLab), max(stratLab), "")

            ct <- 0
            for (denom in c(0, 1000, 500, 250, 100, 50, 25, 10, 5, 2.5, 1, 0.5, 0.01)) {
                ct <- ct+1

                if (denom > max(abs(stratLab)) || (denom<1 && sum(choiceFL)==0)) {
                    outV3 <- c(outV3, "")
                    next
                }
                
                if (sum(choiceFL)==0) {
                    outV3 <- c(outV3, "")
                    choiceDenomCt[ct] <- choiceDenomCt[ct] + 1
                    next
                }
                
                if (denom==0) {
                    perc <- 100 * sum(choiceLabV[choiceFL]==0)/length(choiceLabV)
                    outV3 <- c(outV3, paste ("\\bf", round (perc)))
                    choiceDenomV[ct] <-  choiceDenomV[ct] + perc ; choiceDenomCt[ct] <- choiceDenomCt[ct] + 1
                    choiceFL <- choiceFL & !(choiceLabV==0)
                 
                } else if (denom==0.01) {
                    perc <- 100 * sum(choiceFL)/length(choiceLabV)
                    outV3 <- c(outV3, paste (ifelse (perc>30, "\\bf", ""), round (perc)))
                    choiceDenomV[ct] <-  choiceDenomV[ct] + perc ; choiceDenomCt[ct] <- choiceDenomCt[ct] + 1

                } else {
                    perc <- 100 * sum(choiceLabV[choiceFL] %% denom==0)/length(choiceLabV)
                    outV3 <- c(outV3, paste (ifelse (perc>30, "\\bf", ""), round (perc)))
                    choiceDenomV[ct] <-  choiceDenomV[ct] + perc ; choiceDenomCt[ct] <- choiceDenomCt[ct] + 1
                    choiceFL <- choiceFL & !(choiceLabV %% denom==0)
                }
            }
            
            tabFile <- paste ("tex/lev-distr-", exp, "-", treat, ".3.tex", sep="")
            zz <- file (tabFile, "w")
            cat (paste (outV3, collapse=" & "), file=zz)
            close (zz)
            
        }

        myMin <- function (x) min(as.double(x))
        myMax <- function (x) max(as.double(x))
        myRange <- function (x) paste (myMin(x), "--", myMax(x), sep="")
        myMean <- function (x) round(mean(as.double(x)), digits=2)
        #
        outV3 <- rep ("", 12)
        outV3[1] <- myRange(eESLabelV)
        outV3[2] <- paste (sort(unique(eESRelLevV)), collapse=",")
        outV3[3] <- myRange(eESRelFreqV)
        outV3[5] <- myMean(eQ100V)
        outV3[6] <- myMean(eQ90V)
        outV3[7] <- myMean(eEntrV)
        outV3[8] <- myMean(eExpEntrV)
        outV3[10] <- myRange(eTopOptionsV)
        outV3[11] <- myMean(eTopRelFreqV)
        outV3[12] <- myMean(eOtherRelFreqV)

        tabFile <- paste ("tex/lev-distr-", exp, ".2b.tex", sep="")
        zz <- file (tabFile, "w")
        cat (paste (outV3, collapse=" & "), file=zz)
        close (zz)
        
        tabFile <- paste ("tex/lev-distr-", exp, ".2c.tex", sep="")
        zz <- file (tabFile, "w")
        cat (paste (outV3[5:12], collapse=" & "), file=zz)
        close (zz)
        
        # Choice distribution pooled across treatments, for each experiment
        averNums <- choiceDenomV / pmax(1, choiceDenomCt)
        averNums <- round (averNums / sum(averNums) * 100)
        averNums[choiceDenomCt==0] <- ""

        for (i in 2:length(averNums)) {
            if (averNums[i]=="") next
            if (as.numeric(averNums)[i]>30) averNums[i] <- paste ("\\bf", averNums[i])
        }
        averNums[1] <- paste ("\\bf", averNums[1])
        
        tabFile <- paste ("tex/lev-distr-", exp, ".3b.tex", sep="")
        zz <- file (tabFile, "w")
        cat (paste (averNums, collapse=" & "), file=zz)
        close (zz)
    }

    # Print correllations

    # NumOptions with expEntr
    # relLevEq with expEntr
    # numOptMaxLev with expEntr
    # relLevEq with freqEqSplit
    # expEntr with lev1-rel-freq
    # NumOptions with lev1-rel-freq
    # relLevEq with lev1-rel-freq
    # numOptMaxLev with lev1-rel-freq

    brks <- c(8, 18, 24, 33)
    allCorrV <- c()
    allCorrPV <- c()

    allCorrV2 <- c()
    allCorrPV2 <- c()
    
    for (t in 1:21) {
        
        if (t==1) { v1 <- numChoicesV ; v2 <- expEntrV }
        if (t==2) { v1 <- relLevEqV; v2 <- expEntrV }
        if (t==3) { v1 <- numOptMaxLevV; v2 <- expEntrV }
        if (t==4) { v1 <- expEntrV; v2 <- freqEqSplitV }
        if (t==5) { v1 <- expEntrV; v2 <- lev1V }
        if (t==6) { v1 <- expEntrV; v2 <- min90V }
        if (t==7) { v1 <- relLevEqV; v2 <- lev1V }
        if (t==8) { v1 <- numOptMaxLevV; v2 <- lev1V }
        
        if (t==10) { v1 <- numChoicesV; v2 <- -log (pmax(1e-3, levProbV1/levProbV2)) }
        if (t==11) { v1 <- expEntrV; v2 <- -log (pmax(1e-3, levProbV1/levProbV2)) }
        if (t==12) { v1 <- relLevEqV; v2 <- -log (pmax(1e-3, levProbV1/levProbV2)) }
        if (t==13) { v1 <- freqEqSplitV; v2 <- -log (pmax(1e-3, levProbV1/levProbV2)) }

        if (t==14) { v1 <- numChoicesV; v2 <- -log (pmax(1e-3, levProbV1a/levProbV2a)) }
        if (t==15) { v1 <- expEntrV; v2 <- -log (pmax(1e-3, levProbV1a/levProbV2a)) }
        if (t==16) { v1 <- relLevEqV; v2 <- -log (pmax(1e-3, levProbV1a/levProbV2a)) }
        if (t==17) { v1 <- freqEqSplitV; v2 <- -log (pmax(1e-3, levProbV1a/levProbV2a)) }

        if (t==18) { v1 <- numChoicesV; v2 <- -log (pmax(1e-3, levProbV1b/levProbV2b)) }
        if (t==19) { v1 <- expEntrV; v2 <- -log (pmax(1e-3, levProbV1b/levProbV2b)) }
        if (t==20) { v1 <- relLevEqV; v2 <- -log (pmax(1e-3, levProbV1b/levProbV2b)) }
        if (t==21) { v1 <- freqEqSplitV; v2 <- -log (pmax(1e-3, levProbV1b/levProbV2b)) }

        outV <- c()
        
        for (s in 1:4) {
            if (t>8 && s>3) next
            
            if (s==1 && t<=8) { a1 <- 1 ; a2 <- 24 }      # AM02, HJ06, CHST07
            if (s==1 && t>8) { a1 <- 1 ; a2 <- 18 }      # AM02, HJ06
            if (s==2) { a1 <- 1 ; a2 <- 8 }       # AM02
            if (s==3) { a1 <- 9 ; a2 <- 18 }      # HJ06
            if (s==4) { a1 <- 19 ; a2 <- 24 }     # CHST07
            if (s==5) { a1 <- 25 ; a2 <- 33 }     # Taking
        
            v <- cor (v1[a1:a2], v2[a1:a2], method="spearman")
            if (s>=1) {
                p <- cor.test (v1[a1:a2], v2[a1:a2], method="spearman")$p.value
                r2 <- cor (v1[a1:a2], log (pmax(1e-3, v2[a1:a2])), method="spearman")
                p2 <- cor.test (v1[a1:a2], log (pmax(1e-3, v2[a1:a2])), method="spearman")$p.value
            }

            if (t==2323) XXX

            if (is.na(p) || p>=0.05) stars <- ""
            else if (abs(p)<0.005) stars <- "^{**}"
            else if (abs(p)<0.05) stars <- "^{*}"

            cat ("t", t, "\ts", s, "\n")

            outV <- c (outV, paste ("$", round(v, digits=2), stars, "$", sep=""))
            
            if (s==1) {
                allCorrV <- c(allCorrV, paste ("$", round(v, digits=3), "$", sep=""))
                allCorrPV <- c(allCorrPV, paste ("$", round(p, digits=3), "$", sep=""))
                allCorrV2 <- c(allCorrV2, paste ("$", round(r2, digits=3), "$", sep=""))
                allCorrPV2 <- c(allCorrPV2, paste ("$", round(p2, digits=3), "$", sep=""))
            }

        }
        
        if (t>=1) {
            tabFile <- paste ("tex/cor-relprob-", t, ".tex", sep="")
            zz <- file (tabFile, "w")
            cat (paste (c(allCorrV[t], allCorrPV[t]), collapse=" & "), file=zz)
            close (zz)
            
            tabFile <- paste ("tex/cor-relprob2-", t, ".tex", sep="")
            zz <- file (tabFile, "w")
            cat (paste (c(allCorrV2[t], allCorrPV2[t]), collapse=" & "), file=zz)
            close (zz)
        }
    }

    tabFile <- paste ("tex/cor-prom-", t, ".tex", sep="")
    zz <- file (tabFile, "w")
    cat (paste (outV, collapse=" & "), file=zz)
    close (zz)

    
}


################################################################################################
#
#          Homogeneity   Heterog Pref    Het Pref & Prec    Full Heterog
# Logit
# Prom

if (9 %in% TablesDoBeDone) {

    for (exp in c("pool2", "pool5", "pool7", "pool", "bard", "capp", "fis", "garp", "harr", "list"))
        for (scale in c("R2", "LL", "L0")) 
            for (Moderate in c(2,0,1))
            for (measure in c("BIC", "PW", "EW", "PB1", "PB1a", "PB1e", "PB2", "PB2a", "PB2e", "PB3a", "PB3c", "PB3g")) {
                for (model in c("logit", "prom")) {
                    
                    anzDigits <- ifelse (scale=="R2", 3,1)
                    
                    outV <- c()
                    prevMode <- ""
                    for (MixMode in c(-1,1,3,4)) {
                        m <- loadAdeqMeasure (exp, model, MixMode, Moderate, measure, scale)

                        if (prevMode=="") {
                            if (scale == "R2")
                                outV <- c(outV, "", "", "", "", paste (round(m$meas, digits=anzDigits), sep=""))
                            else 
                                outV <- c(outV, round(m$llMax, digits=1), "", round(m$llMin, digits=1), "",
                                          paste (round(m$meas, digits=anzDigits), sep=""))
                        } else {
                            llTest <- makeLLtest (c(exp, model, prevMode, Moderate), c(exp, model, MixMode, Moderate),
                                                  measure, Nested=TRUE && measure=="BIC")
                            
                            outV <- c(outV, correctRelSign (llTest$relSign, scale),
                                      paste (round(m$meas, digits=anzDigits), sep=""))
                            
#                            if (scale=="R2" && llTest$relSign!="$\\approx$") XXX
                        }
                        
                        prevMode <- MixMode
                    }
                    
                    tabFile <- paste ("tex/incrMods-", exp, "-", scale, "-", Moderate, "-", measure, "-", model, ".tex", sep="")
                    cat (tabFile, "\n")
                    
                    zz <- file (tabFile, "w")
                    cat (paste (outV, collapse=" & "), file=zz)
                    close (zz)

                    zz <- file (paste(tabFile, 2, sep=""), "w")
                    cat (paste (outV[5:length(outV)], collapse=" & "), file=zz)
                    close (zz)
                }
            }
        
    
}
